from silmused.ChecksLayer import ChecksLayer
from silmused.ExecuteLayer import ExecuteLayer
from silmused.tests.DataTest import DataTest
from silmused.tests.StructureTest import StructureTest
from silmused.tests.ConstraintTest import ConstraintTest
from silmused.tests.FunctionTest import FunctionTest
from silmused.tests.IndexTest import IndexTest
from silmused.tests.ProcedureTest import ProcedureTest
from silmused.tests.TriggerTest import TriggerTest
from silmused.tests.ViewTest import ViewTest

_user1 = 123456
_user2 = 123457
_partii_id = 123123
_asula_id = 200
_asula_nimi = 'Test_asula'
_klubi_id = 201
_klubi_nimi = 'Klubi kustutamiseks'

tests = [
    # Prakikum 3
    # Ülesanne 1, 14; 10p
    ChecksLayer(
        title='Prakikum 3 Tabeli Turniirid kontrollid',
        tests=[
            # 1
            StructureTest(
                title='Kas veerg toimumiskoht on kustutatud?',
                name='turniirid',
                column_name='toimumiskoht',
                should_exist=False,
                points=2,
            ),
            # 1
            # StructureTest(
            #    title='Kas veerg asukoht on olemas?',
            #    name='turniirid',
            #    column_name='asukoht',
            #    points=3,
            #    ),
            # 14
            ConstraintTest(
                title='Kas kitsendus ajakontroll on olemas?',
                name='turniirid',
                constraint_name='ajakontroll',
                points=5,
            ),
        ]
    ),
    # Ülesanne 2, 3, 4, 5 ja 8, 9, 12, 17, 18; 9 ül = 35p
    ChecksLayer(
        title='Prakikum 3 Tabeli Isikud kontrollid',
        tests=[
            # 2 ja 4
            StructureTest(
                title='Kas veerg Sisestatud on kustutatud?',
                name='isikud',
                column_name='sisestatud',
                should_exist=False,
                points=3,
            ),
            # 3
            ConstraintTest(
                title='Kas kitsendus un_isikukood on olemas?',
                name='isikud',
                column_name='isikukood',
                constraint_name='un_isikukood',
                constraint_type='UNIQUE',
                points=4,
            ),
            # 5
            ConstraintTest(
                title='Kas kitsendus isikud_un1 on kustutatud?',
                name='isikud',
                constraint_name='isikud_un1',
                constraint_type='UNIQUE',
                should_exist=False,
                points=2,
            ),
            # 8
            DataTest(
                title='Kas isiku Irys perenimi on muudetud?',
                name='isikud',
                column_name='perenimi',
                where="eesnimi = 'Irys'",
                expected_value='Kompvek',
                points=3,
            ),
            # 10
            DataTest(
                title='Kas P-tähega isikud on kustutatud?',
                name='isikud',
                column_name='COUNT(*)',
                where="eesnimi LIKE 'P%' AND RIGHT(eesnimi,1)='P'",
                expected_value=0,
                points=1,
            ),
            # 12
            StructureTest(
                title='Kas veerg klubis on olemas?',
                name='isikud',
                column_name='klubis',
                points=10,
            ),
            # 17
            DataTest(
                title='Kas Siim Susi on klubis Laudnikud?',
                name='isikud',
                column_name='COUNT(*)',
                where="eesnimi = 'Siim' AND perenimi = 'Susi' AND klubis = (select id from klubid where nimi = 'Laudnikud')",
                expected_value=1,
                points=5,
            ),
            # 18
            DataTest(
                title='Kas on lisatud 5 liiget klubisse Osav Oda?',
                name='isikud',
                column_name='COUNT(*)',
                where="klubis = (select id from klubid where nimi ilike 'osav oda')",
                expected_value=5,
                points=7,
            ),
        ]
    ),
    # 9; 15p
    ChecksLayer(
        title='Prakikum 3 Õ tähtede vahetamise kontrollid',
        tests=[
            DataTest(
                title='Kas isikute tabelis veerus eesnimi on "õ" tähed vahetatud?',
                name='isikud',
                column_name='eesnimi',
                where="eesnimi LIKE '%õ%'",
                points=5,
            ),
            DataTest(
                title='Kas isikute tabelis veerus perenimi on "õ" tähed vahetatud?',
                name='isikud',
                column_name='perenimi',
                where="perenimi LIKE '%õ%'",
                points=5,
            ),
            DataTest(
                title='Kas turniiride tabelis veerus nimi on "õ" tähed vahetatud?',
                name='turniirid',
                column_name='nimi',
                where="nimi LIKE '%õ%'",
                points=5,
            ),
        ],
    ),
    # Ülesanne 6, 13, 15; 20p
    ChecksLayer(
        title='Prakikum 3 Tabeli Partiid kontrollid',
        tests=[
            # 6
            ConstraintTest(
                title='Kas kitsendus vastavus on olemas?',
                name='partiid',
                constraint_name='vastavus',
                constraint_type='CHECK',
                points=5,
            ),
            # 13
            StructureTest(
                title='Kas veerg Kokkuvote on kustutatud?',
                name='partiid',
                column_name='kokkuvote',
                should_exist=False,
                points=5,
            ),
            # 15
            ConstraintTest(
                title='Kas kitsendus ajakontroll on olemas?',
                name='partiid',
                constraint_name='ajakontroll',
                points=10,
            ),
        ]
    ),
    # Ülesanne 7, 11, 16; 20p
    ChecksLayer(
        title='Prakikum 3 Tabeli Klubid kontrollid',
        tests=[
            # 7
            # StructureTest(
            #    title='Kas veeru asukoha väärtuse maksimum suurus on õige?',
            #    name='klubid',
            #    column_name='asukoht',
            #    expected_character_maximum_length=100,
            #    points=5,
            # ),
            # 11
            DataTest(
                title='Kas klubi osav oda on olemas?',
                name='klubid',
                where="nimi ilike 'Osav oda'",
                points=10,
            ),
            # 16
            # DataTest(
            #    title='Kas klubi Valge mask asukoht on Valga?',
            #    name='klubid',
            #    column_name='asukoht',
            #    where="nimi = 'Valge Mask'",
            #    expected_value='Valga',
            #    points=5,
            # ),
            # 16, aga üldtesti jaoks pärast praktikum 4
            DataTest(
                title='Kas klubi Valge mask asukoht on Valga?',
                name='klubid',
                column_name='nimi',
                where="nimi = 'Valge Mask' and asula = (select id from asulad where nimi = 'Valga')",
                expected_value='Valge Mask',
                points=5,
            ),
        ]
    ),

    # Praktikum 4
    # Ülesanne 1, 4, 40p
    ChecksLayer(
        title='Prakikum 4 Tabeli Asulad kontrollid',
        tests=[
            StructureTest(
                title='Kas tabel asulad on olemas?',
                name='asulad',
                points=15,
            ),
            StructureTest(
                title='Kas veerg id on olemas?',
                name='asulad',
                column_name='id',
                points=3,
            ),
            StructureTest(
                title='Kas veerg nimi on olemas?',
                name='asulad',
                column_name='nimi',
                points=3,
            ),
            ConstraintTest(
                title='Kas tabeli primaarvõti on olemas?',
                name='asulad',
                constraint_type='PRIMARY KEY',
                points=2,
            ),
            ConstraintTest(
                title='Kas tabelis on unikaalsuse kitsendus?',
                name='asulad',
                constraint_type='UNIQUE',
                points=2,
            ),
            # 4
            DataTest(
                title='Kas tabelis asulad on andmed olemas?',
                name='asulad',
                points=15,
            ),
        ]
    ),
    # Ülesanne 2, 3, 20p
    ChecksLayer(
        title='Prakikum 4 Tabeli Riigid kontrollid',
        tests=[
            StructureTest(
                title='Kas tabel riigid on olemas?',
                name='riigid',
                points=5,
            ),
            StructureTest(
                title='Kas veerg id on olemas?',
                name='riigid',
                column_name='id',
                points=1,
            ),
            StructureTest(
                title='Kas veerg nimi on olemas?',
                name='riigid',
                column_name='nimi',
                points=1,
            ),
            StructureTest(
                title='Kas veerg pealinn on olemas?',
                name='riigid',
                column_name='pealinn',
                points=1,
            ),
            StructureTest(
                title='Kas veerg rahvaarv on olemas?',
                name='riigid',
                column_name='rahvaarv',
                points=1,
            ),
            StructureTest(
                title='Kas veerg pindala on olemas?',
                name='riigid',
                column_name='pindala',
                points=1,
            ),
            StructureTest(
                title='Kas veerg skp_mld on olemas?',
                name='riigid',
                column_name='skp_mld',
                points=1,
            ),
            ConstraintTest(
                title='Kas tabeli primaarvõti on olemas?',
                name='asulad',
                constraint_type='PRIMARY KEY',
                points=2,
            ),
            ConstraintTest(
                title='Kas tabelis on unikaalsuse kitsendus?',
                name='asulad',
                constraint_type='UNIQUE',
                points=2,
            ),
            DataTest(
                title='Kas tabelis riigid on andmed olemas?',
                name='riigid',
                points=5,
            ),
        ]
    ),
    # Ülesanne 5, 6, 7, 9, 40p
    ChecksLayer(
        title='Prakikum 4 Tabeli Klubid kontrollid',
        tests=[
            StructureTest(
                title='Kas veerg asula on olemas?',
                name='klubid',
                column_name='asula',
                points=10,
            ),
            DataTest(
                title='Kas veerus asula on andmed olemas?',
                name='klubid',
                column_name='asula',
                points=10,
            ),
            StructureTest(
                title='Kas veerg asukoht on kustutatud?',
                name='klubid',
                column_name='asukoht',
                should_exist=False,
                points=5,
            ),
            StructureTest(
                title='Kas veerg toimumiskoht on kustutatud?',
                name='klubid',
                column_name='toimumiskoht',
                should_exist=False,
                points=5,
            ),
            ConstraintTest(
                title='Kas tabelis on välisvõti olemas?',
                name='klubid',
                constraint_type='FOREIGN KEY',
                points=10,
            ),

        ]
    ),

    # Kodutöö 3
    # Ülesanne 1 ja 2
    ChecksLayer(
        title='Kodutöö 3 Tabeli Inimesed kontrollid',
        tests=[
            StructureTest(
                title='Kas tabel Inimesed on olemas?',
                name='inimesed',
                points=1,
            ),
            StructureTest(
                title='Kas veerg eesnimi on olemas?',
                name='inimesed',
                column_name='eesnimi',
                points=1,
            ),
            StructureTest(
                title='Kas veerg perenimi on olemas?',
                name='inimesed',
                column_name='perenimi',
                points=1,
            ),
            StructureTest(
                title='Kas veerg sugu on olemas?',
                name='inimesed',
                column_name='sugu',
                points=1,
            ),
            ConstraintTest(
                title='Kas veerul sugu on check kitsendus?',
                name='inimesed',
                column_name='sugu',
                constraint_type='CHECK',
                points=1,
            ),
            StructureTest(
                title='Kas veerg synnipaev on olemas?',
                name='inimesed',
                column_name='synnipaev',
                points=1,
            ),
            StructureTest(
                title='Kas veerg sisestatud on olemas?',
                name='inimesed',
                column_name='sisestatud',
                points=1,
            ),
            StructureTest(
                title='Kas veerg isikukood on olemas?',
                name='inimesed',
                column_name='isikukood',
                points=1,
            ),
            ConstraintTest(
                title='Kas veerul isikukood on unikaalsuse kitsendus?',
                name='inimesed',
                column_name='isikukood',
                constraint_type='UNIQUE',
                points=1,
            ),
            DataTest(
                title='Kas tabelis inimesed on andmed olemas?',
                name='inimesed',
                points=1,
            ),
        ]
    ),
    # Ülesande 3, 4, 5, 6 kontroll
    ChecksLayer(
        title='Kodutöö 3 Tabeli Turniirid kontrollid',
        tests=[
            # Ülesande 3
            StructureTest(
                title='Kas veerg asula on olemas?',
                name='turniirid',
                column_name='asula',
                points=1,
            ),
            # Ülesande 4
            DataTest(
                title='Kas veergu asula on väärtused sisestatud?',
                name='turniirid',
                column_name='asula',
                points=1,
            ),
            # Ülesande 5
            ConstraintTest(
                title='Kas tabelis on välisvõti olemas?',
                name='turniirid',
                constraint_type='FOREIGN KEY',
                points=1,
            ),
            # Ülesande 6 kontrollid
            StructureTest(
                title='Kas veerg toimumiskoht on kustutatud?',
                name='turniirid',
                column_name='toimumiskoht',
                should_exist=False,
                points=0.5,
            ),
            StructureTest(
                title='Kas veerg asukoht on kustutatud?',
                name='turniirid',
                column_name='asukoht',
                should_exist=False,
                points=0.5,
            ),
        ]
    ),
    # Kodutöö 4 kontrollid
    ChecksLayer(
        title='Kodutöö 4 vaate v_turniiripartiid kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_turniiripartiid on olemas?',
                name='v_turniiripartiid',
                points=1,
            ),
            ViewTest(
                title='Kas veerg turniir_nimi on olemas?',
                name='v_turniiripartiid',
                column_name='turniir_nimi',
                points=1,
            ),
            ViewTest(
                title='Kas veerg toimumiskoht on olemas?',
                name='v_turniiripartiid',
                column_name='toimumiskoht',
                points=1,
            ),
            ViewTest(
                title='Kas veerg partii_id on olemas?',
                name='v_turniiripartiid',
                column_name='partii_id',
                points=1,
            ),
            ViewTest(
                title='Kas veerg partii_algus on olemas?',
                name='v_turniiripartiid',
                column_name='partii_algus',
                points=1,
            ),
            ViewTest(
                title='Kas veerg partii_lopp on olemas?',
                name='v_turniiripartiid',
                column_name='partii_lopp',
                points=1,
            ),
            ViewTest(
                title='Kas veerg kes_voitis on olemas?',
                name='v_turniiripartiid',
                column_name='kes_voitis',
            ),
            DataTest(
                title='Kas vaate andmete kogus on õige?',
                name='v_turniiripartiid',
                column_name='COUNT(*)',
                expected_value=299,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas partii 270 võitja tulemus on õige?',
                name='v_turniiripartiid',
                column_name='LOWER(kes_voitis)',
                where='partii_id = 270',
                expected_value='valge',
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas partii 241 võitja tulemus on õige?',
                name='v_turniiripartiid',
                column_name='LOWER(kes_voitis)',
                where='partii_id = 241',
                expected_value='must',
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas partii 193 sai viigi?',
                name='v_turniiripartiid',
                column_name='LOWER(kes_voitis)',
                where='partii_id = 193',
                expected_value='viik',
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas veerg toimumiskoht sisaldab andmeid?',
                name='v_turniiripartiid',
                column_name='COUNT(*)',
                where='toimumiskoht IS NOT NULL',
                isView=True,
                points=1,
            ),
        ]
    ),
    ChecksLayer(
        title='Kodutöö 4 vaate v_klubipartiikogused kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_klubipartiikogused on olemas?',
                name='v_klubipartiikogused',
                points=1,
            ),
            ViewTest(
                title='Kas veerg klubi_nimi on olemas?',
                name='v_klubipartiikogused',
                column_name='klubi_nimi',
                points=1,
            ),
            ViewTest(
                title='Kas veerg partiisid on olemas?',
                name='v_klubipartiikogused',
                column_name='partiisid',
                points=1,
            ),
            DataTest(
                title='Kas vaate andmete kogus on õige?',
                name='v_klubipartiikogused',
                column_name='COUNT(*)',
                expected_value=12,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas vaate partiide summa kokku on õige?',
                name='v_klubipartiikogused',
                column_name='SUM(partiisid)',
                expected_value=571,
                isView=True,
                points=1,
            ),
        ]
    ),

    ChecksLayer(
        title='Kodutöö 4 vaate v_keskminepartii kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_keskminepartii on olemas?',
                name='v_keskminepartii',
                points=1,
            ),
            ViewTest(
                title='Kas veerg turniiri_nimi on olemas?',
                name='v_keskminepartii',
                column_name='turniiri_nimi',
                points=1,
            ),
            ViewTest(
                title='Kas veerg keskmine_partii on olemas?',
                name='v_keskminepartii',
                column_name='keskmine_partii',
                points=1,
            ),
            DataTest(
                title='Kas vaate andmete kogus on õige?',
                name='v_keskminepartii',
                column_name='COUNT(*)',
                expected_value=5,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas turniiril Plekkkarikas 2010 keskmine partii arv on õige?',
                name='v_keskminepartii',
                column_name='ROUND(keskmine_partii, 3)',
                where="turniiri_nimi = 'Plekkkarikas 2010'",
                expected_value=23.765,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas turniiril Kolme klubi kohtumine keskmine partii arv on õige?',
                name='v_keskminepartii',
                column_name='ROUND(keskmine_partii, 2)',
                where="turniiri_nimi = 'Kolme klubi kohtumine'",
                expected_value=23.04,
                isView=True,
                points=1,
            ),
        ]
    ),
    ChecksLayer(
        title='Kodutöö 4 materialiseeritud vaate mv_partiide_arv_valgetega kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade mv_partiide_arv_valgetega on olemas?',
                name='mv_partiide_arv_valgetega',
                isMaterialized=True,
                points=1,
            ),
            DataTest(
                title='Kas vaate andmete kogus on õige?',
                name='mv_partiide_arv_valgetega',
                column_name='COUNT(*)',
                expected_value=85,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas leidub partii ühe mängijaga?',
                name='mv_partiide_arv_valgetega',
                column_name='COUNT(*)',
                where="eesnimi = 'Tarmo' AND perenimi = 'Kooser'",
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas vaate kõige väikseim väärtus on õige?',
                name='mv_partiide_arv_valgetega',
                column_name='MIN(partiisid_valgetega)',
                expected_value=0,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas vaate kõige suurem väärtus on õige?',
                name='mv_partiide_arv_valgetega',
                column_name='MAX(partiisid_valgetega)',
                expected_value=14,
                isView=True,
                points=1,
            )
        ]
    ),
    # Praktikum 9 100p
    # 1; 2p
    ChecksLayer(
        title='Praktikum 9 vaate v_isikudklubid kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_isikudklubid on olemas?',
                name='v_isikudklubid',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_isikudklubid',
                isView=True,
                points=1,
            )
        ],
    ),
    # 2; 2p
    ChecksLayer(
        title='Praktikum 9 vaate v_partiid kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_partiid on olemas?',
                name='v_partiid',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_partiid',
                isView=True,
                points=1,
            )
        ],
    ),
    # 5; 11p
    ChecksLayer(
        title='Praktikum 9 vaate v_partiidpisi kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_isikudklubid on olemas?',
                name='v_partiidpisi',
                points=1,
            ),
            ViewTest(
                title='Kas veerg id on olemas?',
                name='v_partiidpisi',
                column_name='id',
                points=1,
            ),
            ViewTest(
                title='Kas veerg valge_mangija on olemas?',
                name='v_partiidpisi',
                column_name='valge_mangija',
                points=1,
            ),
            ViewTest(
                title='Kas veerg valge_punkt on olemas?',
                name='v_partiidpisi',
                column_name='valge_punkt',
                points=1,
            ),
            ViewTest(
                title='Kas veerg must_mangija on olemas?',
                name='v_partiidpisi',
                column_name='must_mangija',
                points=1,
            ),
            ViewTest(
                title='Kas veerg must_punkt on olemas?',
                name='v_partiidpisi',
                column_name='must_punkt',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_partiidpisi',
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas partiis 1 on valge ja musta mängijate nimed õigel kujul?',
                name='v_partiidpisi',
                column_name='COUNT(*)',
                where="id = 1 and valge_mangija = 'Katrin Kask' and must_mangija = 'Malle Maasikas'",
                expected_value=1,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas partiis 61 valge mängija sai 1 punkti?',
                name='v_partiidpisi',
                column_name='round(valge_punkt,0)',
                where="id = 61 and valge_mangija = 'Jelena Pirn'",
                expected_value=1,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas partiis 61 must mängija sai 0 punkti?',
                name='v_partiidpisi',
                column_name='round(must_punkt,0)',
                where="id = 61 and must_mangija = 'Henno Hiis'",
                expected_value=0,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas partiis 83 saadi viigi eest 0.5 punkti?',
                name='v_partiidpisi',
                column_name='COUNT(*)',
                where="id = 83 and must_punkt = 0.5 and valge_punkt = 0.5",
                expected_value=1,
                isView=True,
                points=1,
            ),
        ],
    ),
    # 6; 20p - total 35
    ChecksLayer(
        title='Praktikum 9 vaate v_punktid kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_punktid on olemas?',
                name='v_punktid',
                points=1,
            ),
            ViewTest(
                title='Kas veerg partii on olemas?',
                name='v_punktid',
                column_name='partii',
                points=1,
            ),
            ViewTest(
                title='Kas veerg turniir on olemas?',
                name='v_punktid',
                column_name='turniir',
                points=1,
            ),
            ViewTest(
                title='Kas veerg mangija on olemas?',
                name='v_punktid',
                column_name='mangija',
                points=1,
            ),
            ViewTest(
                title='Kas veerg varv on olemas?',
                name='v_punktid',
                column_name='varv',
                points=1,
            ),
            ViewTest(
                title='Kas veerg punkt on olemas?',
                name='v_punktid',
                column_name='punkt',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_punktid',
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas mängija on V värvi?',
                name='v_punktid',
                column_name='varv',
                where="partii=51 and mangija=198",
                expected_value='V',
                isView=True,
                points=2,
            ),
            DataTest(
                title='Kas partii kaotaja sai 0 punkti?',
                name='v_punktid',
                column_name='round(punkt,0)',
                where="partii=51 and mangija=198",
                expected_value=0,
                isView=True,
                points=3,
            ),
            DataTest(
                title='Kas mängija on M värvi?',
                name='v_punktid',
                column_name='varv',
                where="partii=51 and mangija=82",
                expected_value='M',
                isView=True,
                points=2,
            ),
            DataTest(
                title='Kas partii võitja sai 1 punkti?',
                name='v_punktid',
                column_name='round(punkt,0)',
                where="partii=51 and mangija=82",
                expected_value=1,
                isView=True,
                points=3,
            ),
            DataTest(
                title='Kas viigi korral saadi 0.5 punkti?',
                name='v_punktid',
                column_name='round(punkt,1)',
                where="partii=54 and mangija=75",
                expected_value=0.5,
                isView=True,
                points=3,
            ),
        ],
    ),
    # 7; 25p -- total 60
    ChecksLayer(
        title='Praktikum 9 vaate v_edetabelid kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_edetabelid on olemas?',
                name='v_edetabelid',
                points=7,
            ),
            ViewTest(
                title='Kas veerg id on olemas?',
                name='v_edetabelid',
                column_name='id',
                points=1,
            ),
            ViewTest(
                title='Kas veerg mangija on olemas?',
                name='v_edetabelid',
                column_name='mangija',
                points=1,
            ),
            ViewTest(
                title='Kas veerg synniaeg on olemas?',
                name='v_edetabelid',
                column_name='synniaeg',
                points=1,
            ),
            ViewTest(
                title='Kas veerg ranking on olemas?',
                name='v_edetabelid',
                column_name='ranking',
                points=1,
            ),
            ViewTest(
                title='Kas veerg klubi on olemas?',
                name='v_edetabelid',
                column_name='klubi',
                points=1,
            ),
            ViewTest(
                title='Kas veerg turniir on olemas?',
                name='v_edetabelid',
                column_name='turniir',
                points=1,
            ),
            ViewTest(
                title='Kas veerg punkte on olemas?',
                name='v_edetabelid',
                column_name='punkte',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_edetabelid',
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas on turniiril 42 kõige suurem punktide arv õige?',
                name='v_edetabelid',
                column_name='round(MAX(punkte),0)',
                where='turniir=42',
                expected_value=7,
                isView=True,
                points=5,
            ),
            DataTest(
                title='Kas on turniiril 41 kõige väiksem punktide arv õige?',
                name='v_edetabelid',
                column_name='round(MIN(punkte),1)',
                where='turniir=41',
                expected_value=0.5,
                isView=True,
                points=5,
            ),
        ],
    ),
    # 10; 10p -- total 70p
    ChecksLayer(
        title='Praktikum 9 vaate mv_edetabelid kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade mv_edetabelid on olemas?',
                name='mv_edetabelid',
                isMaterialized=True,
                points=5,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='mv_edetabelid',
                isView=True,
                points=5,
            )
        ],
    ),
    # 11 ja 12; 20p -- total 90
    ChecksLayer(
        title='Praktikum 9 vaate v_klubi54 kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_klubi54 on olemas?',
                name='v_klubi54',
                points=5,
            ),
            ViewTest(
                title='Kas veerg eesnimi on olemas?',
                name='v_klubi54',
                column_name='eesnimi',
                points=1,
            ),
            ViewTest(
                title='Kas veerg perenimi on olemas?',
                name='v_klubi54',
                column_name='perenimi',
                points=1,
            ),
            ViewTest(
                title='Kas veerg synniaeg on olemas?',
                name='v_klubi54',
                column_name='synniaeg',
                points=1,
            ),
            ViewTest(
                title='Kas veerg ranking on olemas?',
                name='v_klubi54',
                column_name='ranking',
                points=1,
            ),
            ViewTest(
                title='Kas veerg klubi_id on olemas?',
                name='v_klubi54',
                column_name='klubi_id',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_klubi54',
                isView=True,
                points=5,
            ),
            # 12 insert
            DataTest(
                title='Kas Piibe Leht on lisatud vaate kaudu?',
                name='v_klubi54',
                column_name='COUNT(*)',
                where="eesnimi = 'Piibe' and perenimi = 'Leht'",
                expected_value=1,
                isView=True,
                points=5,
            ),
        ],
    ),
    # 13; 10p -- total 100
    ChecksLayer(
        title='Praktikum 9 vaate v_maletaht kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_maletaht on olemas?',
                name='v_maletaht',
                points=1,
            ),
            ViewTest(
                title='Kas veerg id on olemas?',
                name='v_maletaht',
                column_name='id',
                points=1,
            ),
            ViewTest(
                title='Kas veerg eesnimi on olemas?',
                name='v_maletaht',
                column_name='eesnimi',
                points=1,
            ),
            ViewTest(
                title='Kas veerg perenimi on olemas?',
                name='v_maletaht',
                column_name='perenimi',
                points=1,
            ),
            ViewTest(
                title='Kas veerg isikukood on olemas?',
                name='v_maletaht',
                column_name='isikukood',
                points=1,
            ),
            ViewTest(
                title='Kas veerg klubis on olemas?',
                name='v_maletaht',
                column_name='klubis',
                points=1,
            ),
            ViewTest(
                title='Kas veerg synniaeg on olemas?',
                name='v_maletaht',
                column_name='synniaeg',
                points=1,
            ),
            ViewTest(
                title='Kas veerg sugu on olemas?',
                name='v_maletaht',
                column_name='sugu',
                points=1,
            ),
            ViewTest(
                title='Kas veerg ranking on olemas?',
                name='v_maletaht',
                column_name='ranking',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_maletaht',
                column_name='COUNT(*)',
                expected_value=9,
                isView=True,
                points=1,
            )
        ],
    ),
    # Kodutöö 5 kontrollid
    # 25p
    ChecksLayer(
        title='Kodutöö 5 funktsiooni f_vanus kontrollid',
        tests=[
            FunctionTest(
                title='Kas kuupäevaga 09.09.2000 on vanus õige?',
                name='f_vanus',
                arguments=['09.09.2000'],
                expected_value=23,
                points=12,
            ),
            FunctionTest(
                title='Kas kuupäevaga 01.01.2000 on vanus õige?',
                name='f_vanus',
                arguments=['01.01.2000'],
                expected_value=24,
                points=13,
            ),
        ]
    ),
    # 25p
    ChecksLayer(
        title='Kodutöö 5 funktsiooni f_klubiranking kontrollid',
        tests=[
            FunctionTest(
                title='Kas klubi Ajurebend ranking on õige?',
                name='f_klubiranking',
                arguments=[54],
                expected_value=1279.6,
                points=12,
            ),
            FunctionTest(
                title='Kas klubi Musta kivi kummardajad ranking on õige?',
                name='f_klubiranking',
                arguments=[59],
                expected_value=1407.0,
                points=13,
            ),
        ]
    ),
    # 25p
    ChecksLayer(
        title='Kodutöö 5 funktsiooni f_top10 kontrollid',
        tests=[
            FunctionTest(
                title='Kas on õige tulemuste arv?',
                name='f_top10',
                arguments=[44],
                expected_count=10,
                points=10,
            ),
            FunctionTest(
                title='Kas leidub mangija Murakas?',
                name='f_top10',
                where="mangija LIKE 'Murakas%'",
                arguments=[44],
                points=5,
            ),
            FunctionTest(
                title='Kas mangijal on õige paunktide arv?',
                name='f_top10',
                column_name='round(punkte,1)',
                where="mangija LIKE 'Muld%'",
                expected_value=5.5,
                arguments=[47],
                points=10,
            ),
        ]
    ),
    # 25p
    ChecksLayer(
        title='Kodutöö 5 protseduuri sp_uus_turniir kontrollid',
        tests=[
            ProcedureTest(
                title='Kas on õige lõppkuupäev ühe päevasel turniiril?',
                name='sp_uus_turniir',
                arguments=['Haapsalu Meister', '02.02.2022', 1, 'Haapsalu'],
                number_of_parameters=4,
                pre_query="DELETE FROM turniirid WHERE nimi='Haapsalu Meister'",
                after_query="select * from turniirid where nimi = 'Haapsalu Meister' and (loppkuupaev = '02.02.2022' or loppkuupaev = '2022-02-22')",
                points=11,
            ),
            DataTest(
                title='Kas turniiriga lisati ka uus asula?',
                name='asulad',
                where="nimi = 'Haapsalu'",
                points=2,
            ),
            ProcedureTest(
                title='Kas on õige lõppkuupäev kahe päevasel turniiril?',
                name='sp_uus_turniir',
                arguments=['Tartu Meister', '02.02.2022', 2, 'Tartu'],
                number_of_parameters=4,
                pre_query="DELETE FROM turniirid WHERE nimi='Tartu Meister'",
                after_query="select * from turniirid where nimi = 'Tartu Meister' and (loppkuupaev = '03.02.2022' or loppkuupaev = '2022-02-03')",
                points=12,
            ),
        ]
    ),
    # Praktikum 10
    # 1, 10p
    ChecksLayer(
        title='Praktikum 10 funktsiooni f_mangija_punktid_turniiril kontrollid',
        tests=[
            FunctionTest(
                title='Kas mängija punktid turniiril on õige?',
                name='f_mangija_punktid_turniiril',
                arguments=[92, 42],
                number_of_parameters=2,
                expected_value=5.5,
                points=10,
            ),
        ],
    ),
    # 2.1 5p
    ChecksLayer(
        title='Praktikum 10 funktsiooni f_nimi kontrollid',
        tests=[
            FunctionTest(
                title='Kas nimekuju on õige?',
                name='f_nimi',
                arguments=['test', 'kesk'],
                number_of_parameters=2,
                expected_value='kesk, test',
                points=5,
            ),
        ],
    ),
    # 2.2 5p
    ChecksLayer(
        title='Praktikum 10 funktsiooni f_klubisuurus kontrollid',
        tests=[
            FunctionTest(
                title='Kas klubisuurus on õige?',
                name='f_klubisuurus',
                arguments=[50],
                number_of_parameters=1,
                expected_value=9,
                points=5,
            ),
        ],
    ),
    # 3, 10p (30p)
    ChecksLayer(
        title='Praktikum 10 funktsiooni f_mangija_koormus kontrollid',
        tests=[
            FunctionTest(
                title='Kas mängija koormus on õige?',
                name='f_mangija_koormus',
                arguments=[73],
                number_of_parameters=1,
                expected_value=18,
                points=10,
            ),
        ],
    ),
    # 4, 10p (40p)
    ChecksLayer(
        title='Praktikum 10 funktsiooni f_mangija_voite_turniiril kontrollid',
        tests=[
            FunctionTest(
                title='Kas mängija võitude arv on õige?',
                name='f_mangija_voite_turniiril',
                arguments=[197, 43],
                number_of_parameters=2,
                expected_value=3,
                points=5,
            ),
            FunctionTest(
                title='Kas mängija võitude arv on 0?',
                name='f_mangija_voite_turniiril',
                arguments=[75, 44],
                number_of_parameters=2,
                expected_value=0,
                points=5,
            ),
        ],
    ),
    # 5.1, 10 (50p)
    ChecksLayer(
        title='Praktikum 10 funktsiooni f_mangija_viike_turniiril kontrollid',
        tests=[
            FunctionTest(
                title='Kas mängija viikide arv on õige?',
                name='f_mangija_viike_turniiril',
                arguments=[197, 43],
                number_of_parameters=2,
                expected_value=1,
                points=5,
            ),
            FunctionTest(
                title='Kas mängija viikide arv on 0?',
                name='f_mangija_viike_turniiril',
                arguments=[75, 44],
                number_of_parameters=2,
                expected_value=0,
                points=5,
            ),
        ],
    ),
    # 5.2, 10 (60p)
    ChecksLayer(
        title='Praktikum 10 funktsiooni f_mangija_kaotusi_turniiril kontrollid',
        tests=[
            FunctionTest(
                title='Kas mängija kaotuste arv on õige?',
                name='f_mangija_kaotusi_turniiril',
                arguments=[75, 44],
                number_of_parameters=2,
                expected_value=2,
                points=5,
            ),
            FunctionTest(
                title='Kas mängija kaotuste arv on 0?',
                name='f_mangija_kaotusi_turniiril',
                arguments=[197, 43],
                number_of_parameters=2,
                expected_value=0,
                points=5,
            ),
        ],
    ),
    # 6, 10p (70p)
    ChecksLayer(
        title='Praktikum 10 funktsiooni f_voit_viik_kaotus kontrollid',
        tests=[
            FunctionTest(
                title='Kas on õige tulemuste arv?',
                name='f_voit_viik_kaotus',
                arguments=[44],
                number_of_parameters=1,
                expected_count=63,
                points=4,
            ),
            FunctionTest(
                title='Kas mängijal on õige võitude arv?',
                name='f_voit_viik_kaotus',
                column_name='voite',
                where="id = 193",
                arguments=[44],
                number_of_parameters=1,
                expected_value=1,
                points=2,
            ),
            FunctionTest(
                title='Kas mängijal on õige viikide arv?',
                name='f_voit_viik_kaotus',
                column_name='viike',
                where="id = 193",
                arguments=[44],
                number_of_parameters=1,
                expected_value=1,
                points=2,
            ),
            FunctionTest(
                title='Kas mängijal on õige kaotuste arv?',
                name='f_voit_viik_kaotus',
                column_name='kaotusi',
                where="id = 193",
                arguments=[44],
                number_of_parameters=1,
                expected_value=0,
                points=2,
            ),
        ],
    ),
    # 7, 10p (80p)
    ChecksLayer(
        title='Praktikum 10 protseduuri sp_uus_isik kontrollid',
        tests=[
            ProcedureTest(
                title='Kas uus isik sai lisatud?',
                name='sp_uus_isik',
                arguments=['Uus', 'Isik', 51, 'm'],
                number_of_parameters=4,
                pre_query="DELETE FROM isikud WHERE eesnimi = 'Uus' and perenimi = 'Isik'",
                after_query="SELECT * FROM isikud WHERE eesnimi = 'Uus' and perenimi = 'Isik' and klubis = 51",
                points=8,
            ),
            DataTest(
                title='Kas klubiliikmete arv suurenes?',
                name='isikud',
                column_name='COUNT(*)',
                where="klubis=51",
                expected_value=5,
                points=2,
            ),
            ExecuteLayer("DELETE FROM isikud WHERE eesnimi = 'Uus' and pernimi = 'Isik'"),
        ],
    ),
    # 8, 10p (90p)
    ChecksLayer(
        title='Praktikum 10 funktsiooni f_klubiparimad kontrollid',
        tests=[
            FunctionTest(
                title='Kas on õige kirjete arv?',
                name='f_klubiparimad',
                arguments=['Areng'],
                number_of_parameters=1,
                expected_count=3,
                points=5,
            ),
            FunctionTest(
                title='Kas on õige mängija tulemus?',
                name='f_klubiparimad',
                column_name='ROUND(punktisumma,1)',
                arguments=['Areng'],
                number_of_parameters=1,
                where="isik = 'Põder, Priit'",
                expected_value=4.5,
                points=5,
            ),
        ],
    ),
    # 11, 10p, (100p)
    ChecksLayer(
        title='Praktikum 10 funktsiooni f_infopump kontrollid',
        tests=[
            FunctionTest(
                title='Kas on õige kirjete arv?',
                name='f_infopump',
                expected_count=108,
                points=10,
            ),
        ]
    ),
    # Kodutöö 6 kontrollid
    # 10p + 10p
    ChecksLayer(
        title='Kodutöö 6 indeksite kontrollid',
        tests=[
            IndexTest(
                title='Kas on olemas ix_riiginimi?',
                name='ix_riiginimi',
                points=10,
            ),
            IndexTest(
                title='Kas on olemas ix_suurus?',
                name='ix_suurus',
                points=10,
            ),
        ],
    ),
    # 25p
    ChecksLayer(
        title='Kodutöö 6 trigger tg_partiiaeg kontrollid',
        tests=[
            TriggerTest(
                title='Kas on õige definitsioon tg_partiiaeg?',
                name='tg_partiiaeg',
                arguments=['UPDATE', 'INSERT'],
                action_timing='BEFORE',
                points=10,
            ),
            ExecuteLayer("ALTER TABLE public.partiid DISABLE TRIGGER ALL"),
            ExecuteLayer("ALTER TABLE public.partiid ENABLE TRIGGER tg_partiiaeg"),
            ExecuteLayer("ALTER TABLE public.isikud DISABLE TRIGGER ALL"),
            ExecuteLayer(f"DELETE FROM public.isikud WHERE id in ({_user1},{_user2})"),
            ExecuteLayer(f"INSERT INTO public.isikud (id, eesnimi, perenimi) VALUES ({_user1},'Man', 'Ka')"),
            ExecuteLayer(f"INSERT INTO public.isikud (id, eesnimi, perenimi) VALUES ({_user2},'Kan', 'Ma')"),
            DataTest(
                title='Kas testimiseks õnnestus lisada test_isik1?',
                name='isikud',
                where=f"public.isikud.id = {_user1} and EXISTS(SELECT * FROM information_schema.triggers WHERE trigger_name = 'tg_partiiaeg')",
                points=2,
            ),
            DataTest(
                title='Kas testimiseks õnnestus lisada test_isik2?',
                name='isikud',
                where=f"public.isikud.id = {_user2} and EXISTS(SELECT * FROM information_schema.triggers WHERE trigger_name = 'tg_partiiaeg')",
                points=2,
            ),
            ExecuteLayer(
                f"INSERT INTO public.partiid VALUES (44,'2023-04-22 17:45:24.000','2023-03-22 17:45:24.000',{_user1},{_user2},2,0, {_partii_id})", ),
            DataTest(
                title='Kas testimiseks lisatud isikute partii lõpphetk on õige? Kui sul tuleb count = 0, siis sisestatud kirjel pole lopphetk NULL.',
                name='partiid',
                column_name='lopphetk',
                where=f"valge = {_user1} AND must = {_user2} and EXISTS(SELECT * FROM information_schema.triggers WHERE trigger_name = 'tg_partiiaeg')",
                expected_value='NULL',
                points=11,
            ),
        ],
    ),
    # 25p
    ChecksLayer(
        title='Kodutöö 6 trigger tg_klubi_olemasolu kontrollid',
        tests=[
            ExecuteLayer("ALTER TABLE public.partiid DISABLE TRIGGER ALL"),
            ExecuteLayer("ALTER TABLE public.isikud DISABLE TRIGGER ALL"),
            ExecuteLayer("ALTER TABLE public.isikud ENABLE TRIGGER tg_klubi_olemasolu"),
            ExecuteLayer(f"DELETE FROM public.isikud WHERE id in ({_user1},{_user2})"),
            ExecuteLayer(f"INSERT INTO public.isikud (id, eesnimi, perenimi) VALUES ({_user1},'Man', 'Ka')"),
            TriggerTest(
                title='Kas on õige definitsioon tg_klubi_olemasolu?',
                name='tg_klubi_olemasolu',
                arguments=['UPDATE', 'INSERT'],
                action_timing='AFTER',
                points=10,
            ),
            DataTest(
                title='Kas on olemas klubi Klubitud?',
                name='klubid',
                column_name='COUNT(*)',
                where="lower(nimi) = 'klubitud' and EXISTS(SELECT * FROM information_schema.triggers WHERE trigger_name = 'tg_klubi_olemasolu')",
                points=3,
            ),
            DataTest(
                title='Kas testimiseks õnnestus lisada test_isik1?',
                name='isikud',
                where=f"public.isikud.id = {_user1} and EXISTS(SELECT * FROM information_schema.triggers WHERE trigger_name = 'tg_klubi_olemasolu')",
                points=2,
            ),
            DataTest(
                title='Kas lisatud isik on klubis Klubitud?',
                name='isikud',
                join=f"public.klubid ON public.klubid.id=isikud.klubis",
                where=f"public.isikud.id = {_user1} AND public.klubid.nimi LIKE '%ubitu%' and EXISTS(SELECT * FROM information_schema.triggers WHERE trigger_name = 'tg_klubi_olemasolu')",
                points=10,
            ),
        ]
    ),
    # Praktikum 11
    ChecksLayer(
        title='Praktikum 11 trigeri tg_ajakontroll kontrollid',
        tests=[
            TriggerTest(
                title='Kas on õige definitsioon tg_ajakontroll?',
                name='tg_ajakontroll',
                arguments=['UPDATE', 'INSERT'],
                action_timing='BEFORE',
                points=10,
            ),
        ]
    ),
    ChecksLayer(
        title='Praktikum 11 indeksi ix_nimed kontroll',
        tests=[
            IndexTest(
                title='Kas on olemas indeks ix_nimed?',
                name='ix_nimed',
                points=10,
            ),
        ]
    ),
    ChecksLayer(
        title='Praktikum 11 trigeri tg_riigid kontrollid',
        tests=[
            TriggerTest(
                title='Kas on õige definitsioon tg_riigid?',
                name='tg_riigid',
                arguments=['INSERT'],
                action_timing='BEFORE',
                points=10,
            ),
            ExecuteLayer("ALTER TABLE public.riigid DISABLE TRIGGER ALL"),
            ExecuteLayer("ALTER TABLE public.riigid ENABLE TRIGGER tg_riigid"),
            ExecuteLayer(f"DELETE FROM public.riigid WHERE id in (3000, 3001)"),
            ExecuteLayer(f"INSERT INTO public.riigid (id, nimi, pealinn) VALUES (3000,'Bulgaaria1','Sofia1')"),
            DataTest(
                title='Kas uue nimega riik lisati?',
                name='riigid',
                where="id=3000",
                should_exist=True,
                points=10,
            ),
            ExecuteLayer(f"INSERT INTO public.riigid (id, nimi, pealinn) VALUES (3001,'Bulgaaria1','Sofia1')"),
            DataTest(
                title='Kas sama nimega riiki ei lisatud?',
                name='riigid',
                where="id=3001",
                should_exist=False,
                points=10,
            ),
        ]
    ),
    ChecksLayer(
        title='Praktikum 11 trigeri tg_ajakontroll1 kontrollid',
        tests=[
            TriggerTest(
                title='Kas on õige definitsioon tg_ajakontroll1?',
                name='tg_ajakontroll1',
                arguments=['UPDATE', 'INSERT'],
                action_timing='BEFORE',
                points=10,
            ),
            ExecuteLayer("ALTER TABLE public.partiid DISABLE TRIGGER ALL"),
            ExecuteLayer("ALTER TABLE public.partiid ENABLE TRIGGER tg_ajakontroll1"),
            ExecuteLayer("ALTER TABLE public.isikud DISABLE TRIGGER ALL"),
            ExecuteLayer(f"DELETE FROM public.isikud WHERE id in ({_user1},{_user2})"),
            ExecuteLayer(f"INSERT INTO public.isikud (id, eesnimi, perenimi) VALUES ({_user1},'Man', 'Ka')"),
            ExecuteLayer(f"INSERT INTO public.isikud (id, eesnimi, perenimi) VALUES ({_user2},'Kan', 'Ma')"),
            ExecuteLayer(
                f"INSERT INTO public.partiid VALUES (44,'2023-03-22 17:45:24.000','2023-03-22 17:47:24.000',{_user1},{_user2},2,0, {_partii_id})", ),
            DataTest(
                title='Kas partii lisamine õnnestus?',
                name='partiid',
                where=f"id={_partii_id}",
                points=10,
            ),
            ExecuteLayer(
                f"INSERT INTO partiid (turniir, algushetk, valge, must, valge_tulemus, musta_tulemus) VALUES (41, '2005-01-12 08:05:00.000', 73, 92, 1, 1, {_partii_id + 1})", ),
            DataTest(
                title='Kas partii lisamine ebaõnnestus?',
                name='partiid',
                where=f"id={_partii_id + 1}",
                points=10,
                should_exist=False,
            ),
        ]
    ),
    ChecksLayer(
        title='Praktikum 11 trigeri tg_kustuta_klubi kontrollid',
        tests=[
            TriggerTest(
                title='Kas on õige definitsioon tg_kustuta_klubi?',
                name='tg_kustuta_klubi',
                arguments=['DELETE'],
                action_timing='AFTER',
                points=20,
            ),
            ExecuteLayer("ALTER TABLE public.klubid DISABLE TRIGGER ALL"),
            ExecuteLayer("ALTER TABLE public.klubid ENABLE TRIGGER tg_kustuta_klubi"),
            ExecuteLayer("ALTER TABLE public.asulad DISABLE TRIGGER ALL"),
            ExecuteLayer(f"DELETE FROM public.asulad WHERE id in ({_asula_id})"),
            ExecuteLayer(f"DELETE FROM public.klubid WHERE id in ({_klubi_id})"),
            ExecuteLayer(f"INSERT INTO public.asulad VALUES ({_asula_id},{_asula_nimi})"),
            ExecuteLayer(f"INSERT INTO public.klubid (id, nimi, asula) VALUES ({_klubi_id},{_klubi_nimi},{_asula_id}"),
            ExecuteLayer(f"DELETE FROM klubid WHERE nimi={_klubi_nimi}"),
            DataTest(
                title='Kas klubi kustutamisel kustus ka asula?',
                name='asulad',
                where=f"nimi='{_asula_nimi}'",
                should_exist=False,
                points=1,
            )
        ]
    ),
    ChecksLayer(
        title='Praktikum 11 trigeri tg_partiiaeg1 kontrollid',
        tests=[
            TriggerTest(
                title='Kas on õige definitsioon tg_partiiaeg1?',
                name='tg_partiiaeg1',
                arguments=['UPDATE', 'INSERT'],
                action_timing='BEFORE',
                points=12,
            ),
            ExecuteLayer("ALTER TABLE public.partiid DISABLE TRIGGER ALL"),
            ExecuteLayer("ALTER TABLE public.partiid ENABLE TRIGGER tg_partiiaeg1"),
            ExecuteLayer("ALTER TABLE public.isikud DISABLE TRIGGER ALL"),
            ExecuteLayer(f"DELETE FROM public.isikud WHERE id in ({_user1},{_user2})"),
            ExecuteLayer(f"INSERT INTO public.isikud (id, eesnimi, perenimi) VALUES ({_user1},'Man', 'Ka')"),
            ExecuteLayer(f"INSERT INTO public.isikud (id, eesnimi, perenimi) VALUES ({_user2},'Kan', 'Ma')"),
            ExecuteLayer(
                f"INSERT INTO Partiid(turniir, algushetk, valge, must, id) VALUES (41, '2005-01-22 08:04', {_user1},{_user2},{_partii_id + 2})"),
            DataTest(
                title='Kas partii lisamine ebaõnnestus, kui algusaeg enne turniiri?',
                name='partiid',
                where=f"id={_partii_id + 2}",
                points=2,
                should_exist=False,
            ),
            ExecuteLayer(
                f"INSERT INTO Partiid(turniir, algushetk, valge, must, id) VALUES (41, '2005-01-10 08:04', {_user1},{_user2},{_partii_id + 3})"),
            DataTest(
                title='Kas partii lisamine ebaõnnestus, kui algusaeg peale turniiri?',
                name='partiid',
                where=f"id={_partii_id + 3}",
                points=2,
                should_exist=False,
            ),
            ExecuteLayer(
                f"INSERT INTO Partiid(turniir, algushetk, lopphetk, valge, must, id) VALUES (41, '2005-01-12 08:04', '2005-01-22 09:10', {_user1},{_user2},{_partii_id + 4})"),
            DataTest(
                title='Kas partii lisamine ebaõnnestus, kui lõppaeg enne turniiri?',
                name='partiid',
                where=f"id={_partii_id + 4}",
                points=2,
                should_exist=False,
            ),
            ExecuteLayer(
                f"INSERT INTO Partiid(turniir, algushetk, lopphetk, valge, must, id) VALUES (41, '2005-01-12 08:04', '2005-01-01 09:10', {_user1},{_user2},{_partii_id + 5})"),
            DataTest(
                title='Kas partii lisamine ebaõnnestus, kui lõppaeg peale turniiri?',
                name='partiid',
                where=f"id={_partii_id + 5}",
                points=2,
                should_exist=False,
            ),
        ]
    ),
]


from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest

tests = [
    ChecksLayer(
        title='Ülesanne 1 Päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg lavanimi?',
                name='query_test',
                column_name='lavanimi',
                points=30,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=1,
                points=30,
            ),
            QueryDataTest(
                title='Kas puuduva synniajga on Mariko?',
                name='query_test',
                column_name='lavanimi',
                where="test_id=1",
                expected_value="Mariko",
                points=40,
            ),
        ]
    ),
]


from silmused.tests.TestDefinition import TestDefinition


class IndexTest(TestDefinition):
    def __init__(self, name, title=None, description=None, custom_feedback=None, points=0):
        super().__init__(
            name=name,
            title=title,
            points=points,
            description=description,
            custom_feedback=custom_feedback,
            query=f"SELECT * FROM pg_indexes WHERE indexname = '{name}'",
        )

    def execute(self, cursor):
        cursor.execute(self.query)
        result = cursor.fetchall()
        if self.custom_feedback is None:
            return super().response(
                len(result) > 0,
                {"test_type": "index_test",
                 "test_key": "index_positive_feedback",
                 "params": [self.name]},
                {"test_type": "index_test",
                 "test_key": "index_negative_feedback",
                 "params": [self.name]}
            )
        else:
            return super().response(
                len(result) > 0,
                {"test_type": "index_test",
                 "test_key": "custom_feedback",
                 "params": [self.custom_feedback]},
                {"test_type": "index_test",
                 "test_key": "custom_feedback",
                 "params": [self.custom_feedback]},
            )


from silmused.ChecksLayer import ChecksLayer
from silmused.ExecuteLayer import ExecuteLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest
_user1 = 123456
_user2 = 123457
tests = [
    ChecksLayer(
        title='Ülesanne 3 Päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg ränkingu klass?',
                name='query_test',
                column_name="ränkingu klass",
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg arv?',
                name='query_test',
                column_name='arv',
                points=10,
            ),
            QueryStructureTest(
                title='Kas päringus on kasutatud keelatud elemente?',
                name='query_test',
                elements='LIMIT',
                should_exist=False,
                points=10,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=4,
                points=10,
            ),
            QueryDataTest(
                title='Kas on õige arv mängijaid ränkinguga 1100?',
                name='query_test',
                column_name='arv',
                where='"ränkingu klass" = 1100',
                expected_value=25,
                points=10,
            ),
            QueryDataTest(
                title='Kas on õige ränking viimasel kohal?',
                name='query_test',
                column_name='"ränkingu klass"',
                where='test_id=4',
                expected_value=1300,
                points=10,
            ),
            ExecuteLayer(f"INSERT INTO public.isikud (id, eesnimi, perenimi, ranking) values ({_user1}, 'etest1', 'ptest1', 1600)"),
            ExecuteLayer(f"INSERT INTO public.isikud (id, eesnimi, perenimi, ranking) values ({_user2}, 'etest2', 'ptest2', 1600)"),
            ExecuteLayer("delete from isikud where id = 10;"),
            ExecuteLayer("delete from query_test"),
            ExecuteLayer("insert into query_test select * from query_view;"),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus, kui on lisatud 2 uut isikut ränkiguga 1600?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=5,
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige ränking esimesel kohal kui eemaldada üks isik 1100 ränkiguga?',
                name='query_test',
                column_name='"ränkingu klass"',
                where='test_id=5',
                expected_value=1100,
                points=10,
            ),
        ]
    ),
]


from silmused.tests.TestDefinition import TestDefinition
from silmused.utils import list_to_string


class QueryStructureTest(TestDefinition):
    def __init__(self, name, title=None, column_name=None, arguments=None, should_exist=True, where=None,
                 description=None, custom_feedback=None, elements=None, points=0):

        query = f"SELECT {list_to_string(arguments)[1:-1] if arguments is not None else '*'} FROM information_schema.columns WHERE table_name = '{name}'"

        if column_name is not None:
            if isinstance(column_name, str):
                query += f" AND column_name = '{column_name}'"
            elif isinstance(column_name, list):
                for (index, c_name) in enumerate(column_name):
                    operator = 'AND (' if index == 0 else 'OR'
                    query += f" {operator} column_name = '{c_name}'"
                query += ")"
            else:
                raise AttributeError('Parameter column_name must be list or string')

        if elements is not None:
            query = "SELECT * FROM information_schema.views WHERE table_name = 'query_view'"
            if isinstance(elements, str):
                query += f" AND view_definition ILIKE '%{elements}%'"
            elif isinstance(elements, list):
                for (index, arg) in enumerate(elements):
                    operator = 'AND (' if index == 0 else 'OR'
                    query += f" {operator} view_definition ILIKE '%{arg}%'"
                query += ")"
            else:
                raise AttributeError('Parameter elements must be a list or string')

        super().__init__(
            name=name,
            title=title,
            where=where,
            points=points,
            arguments=arguments,
            description=description,
            query=query,
            should_exist=should_exist,
            custom_feedback=custom_feedback,
            elements=elements,
        )

        self.column_name = column_name
        self.where = where

    def execute(self, cursor):
        if isinstance(self.elements, list):
            self.query = self._check_separately_for_all_elements(cursor)
        if isinstance(self.column_name, list):
            self.query = self._check_separately_for_all_columns(cursor)
        cursor.execute(self.query)
        result = cursor.fetchall()

        if self.elements is not None:
            # TODO if in the future there is a requirement to count specific amount of elements used, then this can be used
            # select (CHAR_LENGTH(lower(view_definition)) - CHAR_LENGTH(REPLACE(lower(view_definition), lower('maletäht'), ''))) / CHAR_LENGTH(lower('maletäht')) as alamparing from information_schema.views where table_name = 'query_view';
            if self.should_exist:
                if self.custom_feedback is None:
                    return super().response(
                        len(result) > 0,
                        {"test_type": "query_structure_test",
                         "test_key": "query_required_elements_positive_feedback",
                         "params": [self.elements]},
                        {"test_type": "query_structure_test",
                         "test_key": "query_required_elements_negative_feedback",
                         "params": [self.elements]},
                    )
                else:
                    return super().response(
                        len(result) > 0,
                        {"test_type": "query_structure_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "query_structure_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
            else:
                if self.custom_feedback is None:
                    return super().response(
                        len(result) == 0,
                        {"test_type": "query_structure_test",
                         "test_key": "query_banned_elements_positive_feedback",
                         "params": [self.elements]},
                        {"test_type": "query_structure_test",
                         "test_key": "query_banned_elements_negative_feedback",
                         "params": [self.elements]},
                    )
                else:
                    return super().response(
                        len(result) == 0,
                        {"test_type": "query_structure_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "query_structure_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
        elif self.should_exist:
            if self.column_name is None:
                if self.custom_feedback is None:
                    return super().response(
                        len(result) > 0,
                        {"test_type": "query_structure_test",
                         "test_key": "query_table_should_exist_positive_feedback",
                         "params": [self.name]},
                        {"test_type": "query_structure_test",
                         "test_key": "query_table_should_exist_negative_feedback",
                         "params": [self.name]},
                    )
                else:
                    return super().response(
                        len(result) > 0,
                        {"test_type": "query_structure_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "query_structure_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
            else:
                if self.custom_feedback is None:
                    return super().response(
                        len(result) > 0,
                        {"test_type": "query_structure_test",
                         "test_key": "query_column_should_exist_positive_feedback",
                         "params": [self.column_name]},
                        {"test_type": "query_structure_test",
                         "test_key": "query_column_should_exist_negative_feedback",
                         "params": [self.column_name]},
                    )
                else:
                    return super().response(
                        len(result) > 0,
                        {"test_type": "query_structure_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "query_structure_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
        else:
            if self.column_name is None:
                if self.custom_feedback is None:
                    return super().response(
                        len(result) == 0,
                        {"test_type": "query_structure_test",
                         "test_key": "query_table_should_not_exist_positive_feedback",
                         "params": [self.name]},
                        {"test_type": "query_structure_test",
                         "test_key": "query_table_should_not_exist_negative_feedback",
                         "params": [self.name]},
                    )
                else:
                    return super().response(
                        len(result) == 0,
                        {"test_type": "query_structure_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "query_structure_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
            else:
                if self.custom_feedback is None:
                    return super().response(
                        len(result) == 0,
                        {"test_type": "query_structure_test",
                         "test_key": "query_column_should_not_exist_positive_feedback",
                         "params": [self.column_name]},
                        {"test_type": "query_structure_test",
                         "test_key": "query_column_should_not_exist_negative_feedback",
                         "params": [self.column_name]},
                    )
                else:
                    return super().response(
                        len(result) == 0,
                        {"test_type": "query_structure_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "query_structure_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )

    def _check_separately_for_all_elements(self, cursor):
        found = []
        for element in self.elements:
            query = f"SELECT * FROM information_schema.views WHERE table_name = 'query_view' AND view_definition ILIKE '%{element}%'"
            cursor.execute(query)
            result = cursor.fetchall()
            if self.should_exist and len(result) == 0:
                found.append(element)
            elif not self.should_exist and len(result) > 0:
                found.append(element)
        if len(found) == 0: return self.query
        query = f"SELECT * FROM information_schema.views WHERE table_name = 'query_view'"
        self.elements = found
        for (index, arg) in enumerate(self.elements):
            operator = 'AND (' if index == 0 else 'OR'
            query += f" {operator} view_definition ILIKE '%{arg}%'"
        query += ")"
        return query

    def _check_separately_for_all_columns(self, cursor):
        found = []
        for column in self.column_name:
            query = f"SELECT * FROM information_schema.columns WHERE table_name = '{self.name}' AND column_name = '{column}'"
            cursor.execute(query)
            result = cursor.fetchall()
            if self.should_exist and len(result) == 0:
                found.append(column)
            elif not self.should_exist and len(result) > 0:
                found.append(column)
        if len(found) == 0: return self.query
        query = f"SELECT * FROM information_schema.columns WHERE table_name = '{self.name}'"
        self.column_name = found
        for (index, c_name) in enumerate(self.column_name):
            operator = 'AND (' if index == 0 else 'OR'
            query += f" {operator} column_name = '{c_name}'"
        query += ")"
        return query



import json
import glob
import os
from datetime import datetime
from string import Template

supported_format = ['json']

class Translator():
    def __init__(self, locale='et', file_format='json'):
        # initialization
        self.data = {}
        self.locale = locale

        current_file_path = os.path.abspath(__file__)
        root_directory = os.path.dirname(current_file_path)
        locale_directory = os.path.join(root_directory, 'locale')
        # check if format is supported
        if file_format in supported_format:
            # get list of files with specific extensions
            files = glob.glob(f'{locale_directory}/'+f'*.{file_format}')
            for fil in files:
                # get the name of the file without extension, will be used as locale name
                loc = os.path.splitext(os.path.basename(fil))[0]
                with open(fil, 'r', encoding='utf8') as f:
                    if file_format == 'json':
                        self.data[loc] = json.load(f)
        #print(self.data['et'])

    def set_locale(self, loc):
        if loc in self.data:
            self.locale = loc
        else:
            print('Invalid locale')

    def translate(self, test_type, test_key, **kwargs):
        if self.locale not in self.data:
            return "Locale not supported: " + self.locale
        if test_type not in self.data[self.locale]:
            return "Test_type not supported: " + test_type + " FOR locale: " + self.locale
        if test_key not in self.data[self.locale][test_type]:
            return "Test_key not supported: " + test_key + " FOR test_type: " + test_type
        # print(self.data[self.locale][test_type])
        text = self.data[self.locale][test_type].get(test_key, test_key)

        # string interpolation
        return Template(text).safe_substitute(**kwargs)


def str_to_datetime(dt_str, format='%Y-%m-%d'):
    return datetime.strptime(dt_str, format)


# def datetime_to_str(dt, format='MMMM dd, yyyy', loc='en'):
#    return format_datetime(dt, format=format, locale=loc)



from silmused.tests.TestDefinition import TestDefinition
from silmused.utils import list_to_string


class ViewTest(TestDefinition):
    def __init__(self, name, title=None, column_name=None, arguments=None, expected_value=None, should_exist=True,
                 where=None, description=None, custom_feedback=None, isMaterialized=False, elements=None, points=0):
        if isMaterialized:
            query = f"SELECT * FROM pg_matviews WHERE matviewname = '{name}'"
        else:
            query = f"SELECT {list_to_string(arguments)[1:-1] if arguments is not None else '*'} FROM information_schema.columns WHERE table_name = '{name}'"

        if column_name is not None:
            if type(column_name) == str:
                query += f" AND column_name = '{column_name}'"
            elif type(column_name) == list:
                for (index, name) in enumerate(column_name):
                    operator = 'AND' if index == 0 else 'OR'
                    query += f" {operator} column_name = '{name}'"
            else:
                raise AttributeError('Parameter column_name must be list or string')

        if elements is not None:
            if not isMaterialized:
                query = f"SELECT * FROM pg_views WHERE viewname = '{name}'"
            if isinstance(elements, str):
                query += f" AND definition ILIKE '%{elements}%'"
            elif isinstance(elements, list):
                for (index, arg) in enumerate(elements):
                    operator = 'AND (' if index == 0 else 'OR'
                    query += f" {operator} definition ILIKE '%{arg}%'"
                query += ")"
            else:
                raise AttributeError('Parameter banned_arguments must be list or string')

        super().__init__(
            name=name,
            title=title,
            where=where,
            points=points,
            arguments=arguments,
            expected_value=expected_value,
            description=description,
            query=query,
            should_exist=should_exist,
            custom_feedback=custom_feedback,
            elements=elements,
        )

        self.column_name = column_name
        self.where = where
        self.isMaterialized = isMaterialized

    def execute(self, cursor):
        if isinstance(self.elements, list):
            self.query = self._check_separately_for_all_elements(cursor)
        if isinstance(self.column_name, list):
            self.query = self._check_separately_for_all_columns(cursor)
        cursor.execute(self.query)
        result = cursor.fetchall()

        if self.isMaterialized:
            if self.elements is not None:
                if self.should_exist:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "view_test",
                             "test_key": "mat_view_required_elements_positive_feedback",
                             "params": [self.elements,self.name]},
                            {"test_type": "query_structure_test",
                             "test_key": "mat_view_required_elements_negative_feedback",
                             "params": [self.elements,self.name]},
                        )
                    else:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "view_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "query_structure_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
                else:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "view_test",
                             "test_key": "mat_view_banned_elements_positive_feedback",
                             "params": [self.elements,self.name]},
                            {"test_type": "query_structure_test",
                             "test_key": "mat_view_banned_elements_negative_feedback",
                             "params": [self.elements,self.name]},
                        )
                    else:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "view_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "query_structure_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
            elif self.should_exist:
                if self.custom_feedback is None:
                    return super().response(
                        len(result) > 0,
                        {"test_type": "view_test",
                         "test_key": "mat_view_should_exist_positive_feedback",
                         "params": [self.name]},
                        {"test_type": "view_test",
                         "test_key": "mat_view_should_exist_negative_feedback",
                         "params": [self.name]},
                    )
                else:
                    return super().response(
                        False,
                        {"test_type": "view_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "view_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
            else:
                if self.custom_feedback is None:
                    return super().response(
                        len(result) == 0,
                        {"test_type": "view_test",
                         "test_key": "mat_view_should_not_exist_positive_feedback",
                         "params": [self.name]},
                        {"test_type": "view_test",
                         "test_key": "mat_view_should_not_exist_negative_feedback",
                         "params": [self.name]},
                    )
                else:
                    return super().response(
                        False,
                        {"test_type": "view_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "view_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
        if self.elements is not None:
            if self.should_exist:
                if self.custom_feedback is None:
                    return super().response(
                        len(result) > 0,
                            {"test_type": "view_test",
                             "test_key": "view_required_elements_positive_feedback",
                             "params": [self.elements,self.name]},
                            {"test_type": "query_structure_test",
                             "test_key": "view_required_elements_negative_feedback",
                             "params": [self.elements,self.name]},
                    )
                else:
                    return super().response(
                        len(result) > 0,
                        {"test_type": "view_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "view_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
            else:
                if self.custom_feedback is None:
                    return super().response(
                        len(result) == 0,
                            {"test_type": "view_test",
                             "test_key": "view_banned_elements_positive_feedback",
                             "params": [self.elements,self.name]},
                            {"test_type": "query_structure_test",
                             "test_key": "view_banned_elements_negative_feedback",
                             "params": [self.elements,self.name]},
                    )
                else:
                    return super().response(
                        len(result) == 0,
                        {"test_type": "view_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "view_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
        if self.expected_value is not None:
            # TODO Is this needed?
            # What is this test checking? result 0,0 is database name, maybe this should check if the value is found in result?
            if self.should_exist:
                if self.custom_feedback is None:
                # print(self.title, result[0][0])
                    return super().response(
                        len(result) != 0 and result[0][0] == self.expected_value,
                        {"test_type": "view_test",
                         "test_key": "expected_value_should_exist_positive_feedback",
                         "params": [self.query]},
                        {"test_type": "view_test",
                         "test_key": "expected_value_should_exist_negative_feedback",
                         "params": [self.query, self.expected_value]},
                    )
                else:
                    return super().response(
                        False,
                        {"test_type": "view_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "view_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
            else:
                if self.custom_feedback is None:
                # print(self.title, result[0][0])
                    return super().response(
                        len(result) == 0 or result[0][0] != self.expected_value,
                        {"test_type": "view_test",
                         "test_key": "expected_value_should_not_exist_positive_feedback",
                         "params": [self.query]},
                        {"test_type": "view_test",
                         "test_key": "expected_value_should_not_exist_negative_feedback",
                         "params": [self.query, self.expected_value]},
                    )
                else:
                    return super().response(
                        False,
                        {"test_type": "view_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "view_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
        else:
            if self.should_exist:
                if self.column_name is None:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "view_test",
                             "test_key": "view_should_exist_positive_feedback",
                             "params": [self.name]},
                            {"test_type": "view_test",
                             "test_key": "view_should_exist_negative_feedback",
                             "params": [self.name]},
                        )
                    else:
                        return super().response(
                            False,
                            {"test_type": "view_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "view_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
                else:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "view_test",
                             "test_key": "column_should_exist_positive_feedback",
                             "params": [self.column_name, self.name]},
                            {"test_type": "view_test",
                             "test_key": "column_should_exist_negative_feedback",
                             "params": [self.column_name, self.name]},
                        )
                    else:
                        return super().response(
                            False,
                            {"test_type": "view_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "view_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
            else:
                if self.column_name is None:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "view_test",
                             "test_key": "view_should_not_exist_positive_feedback",
                             "params": [self.name]},
                            {"test_type": "view_test",
                             "test_key": "view_should_not_exist_negative_feedback",
                             "params": [self.name]},
                        )
                    else:
                        return super().response(
                            False,
                            {"test_type": "view_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "view_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
                else:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "view_test",
                             "test_key": "column_should_not_exist_positive_feedback",
                             "params": [self.column_name, self.name]},
                            {"test_type": "view_test",
                             "test_key": "column_should_not_exist_negative_feedback",
                             "params": [self.column_name, self.name]},
                        )
                    else:
                        return super().response(
                            False,
                            {"test_type": "view_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "view_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )

    def _check_separately_for_all_elements(self, cursor):
        found = []
        for element in self.elements:
            query = f"SELECT * FROM information_schema.views WHERE table_name = 'query_view' AND view_definition ILIKE '%{element}%'"
            cursor.execute(query)
            result = cursor.fetchall()
            if self.should_exist and len(result) == 0:
                found.append(element)
            elif not self.should_exist and len(result) > 0:
                found.append(element)
        if len(found) == 0: return self.query
        if self.isMaterialized:
            query = f"SELECT * FROM pg_matviews WHERE matviewname = '{self.name}'"
        else:
            query = f"SELECT * FROM pg_views WHERE viewname = '{self.name}'"
        self.elements = found
        for (index, arg) in enumerate(self.elements):
            operator = 'AND (' if index == 0 else 'OR'
            query += f" {operator} view_definition ILIKE '%{arg}%'"
        query += ")"
        return query

    def _check_separately_for_all_columns(self, cursor):
        found = []
        for column in self.column_name:
            query = f"SELECT * FROM information_schema.columns WHERE table_name = '{self.name}' AND column_name = '{column}'"
            cursor.execute(query)
            result = cursor.fetchall()
            if self.should_exist and len(result) == 0:
                found.append(column)
            elif not self.should_exist and len(result) > 0:
                found.append(column)
        if len(found) == 0: return self.query
        query = f"SELECT * FROM information_schema.columns WHERE table_name = '{self.name}'"
        self.column_name = found
        for (index, c_name) in enumerate(self.column_name):
            operator = 'AND (' if index == 0 else 'OR'
            query += f" {operator} column_name = '{c_name}'"
        query += ")"
        return query



from silmused.TitleLayer import TitleLayer
from silmused.ChecksLayer import ChecksLayer
from silmused.ExecuteLayer import ExecuteLayer
from silmused.tests.DataTest import DataTest
from silmused.tests.StructureTest import StructureTest
from silmused.tests.ConstraintTest import ConstraintTest
from silmused.tests.FunctionTest import FunctionTest
from silmused.tests.IndexTest import IndexTest
from silmused.tests.ProcedureTest import ProcedureTest
from silmused.tests.TriggerTest import TriggerTest
from silmused.tests.ViewTest import ViewTest

_user1 = 123456
_user2 = 123457
_partii_id = 123123

tests = [
    #TitleLayer('Praktikum 3'),

    ChecksLayer(
        title='Tabeli Isikud kontrollid',
        tests=[
            ConstraintTest(
                title='Kas kitsendus un_isikukood on olemas?',
                name='isikud',
                constraint_name='un1_isikukood',
                constraint_type='UNIQUE',
                points=0.5,
            ),
            ConstraintTest(
                title='Kas kitsendus un_isikukood on kustutatud?',
                name='isikud',
                constraint_name='nimi_unique',
                constraint_type='UNIQUE',
                should_exist=False,
                points=0.25,
            ),
        ]
    ),

    ChecksLayer(
        title='Tabeli Turniirid kontrollid',
        tests=[
            StructureTest(
                title='Kas veerg asukoht on olemas?',
                name='turniirid',
                column_name='asukoht',
                should_exist=False,
                points=2,
                )
            ]
        ),
    ChecksLayer(
        title='Tabeli Partiid kontrollid',
        tests=[
            ConstraintTest(
                title='Kas kitsendus vastavus on olemas?',
                name='partiid',
                constraint_name='vastavus',
                points=1,
                )
            ]
        ),
    ChecksLayer(
        title='Tabeli Klubid kontrollid',
        tests=[
            StructureTest(
                title='Kas veerg asukoht on olemas?',
                name='klubid',
                column_name='asukoht',
                points=1,
            ),
            StructureTest(
                title='Kas veeru asukoha väärtuse maksimum suurus on õige?',
                name='klubid',
                column_name='asukoht',
                expected_character_maximum_length=100,
                points=1,
            ),
            ConstraintTest(
                title='Kas tabeli primaarvõti on olemas?',
                name='klubid',
                constraint_type='PRIMARY KEY',
                points=0.5,
            )
        ]
    ),
    #TitleLayer('Praktikum 4'),
    ChecksLayer(
        title='Tabeli Asulad kontrollid',
        tests=[
            StructureTest(
                title='Kas tabel Asulad on olemas?',
                name='asulad',
            ),
            DataTest(
                title='Kas tabelis on andmed olemas?',
                name='asulad',
            ),
        ]
    ),
    ChecksLayer(
        title='Tabeli Riigid kontrollid',
        tests=[
            StructureTest(
                title='Kas tabel Riigid on olemas?',
                name='riigid',
            ),
        ]
    ),
    ChecksLayer(
        title='Tabeli Klubid kontrollid',
        tests=[
            StructureTest(
                title='Kas veerg asula on olemas?',
                name='klubid',
                column_name='asula',
            ),
            DataTest(
                title='Kas veeru asula andmed on olemas?',
                name='klubid',
                column_name='asula',
            ),
            ConstraintTest(
                title='Kas tabeli primaarvõti on olemas?',
                name='klubid',
                constraint_type='PRIMARY KEY'
            ),
            StructureTest(
                title='Kas veerg asukoht on kustutatud?',
                name='klubid',
                column_name='asukoht',
                should_exist=False,
            ),
        ]
    ),
    #TitleLayer('Kodutöö 3'),
    ChecksLayer(
        title='Tabeli Isikud kontrollid ',
        tests=[
            DataTest(
                title='Kas tabel Isikud on olemas?',
                name='isikud',
                points=1,
            ),
            ConstraintTest(
                title='Kas tabeli primaarvõti on olemas?',
                name='isikud',
                constraint_type='PRIMARY KEY',
                points=1,
            ),
            ConstraintTest(
                title='Kas tabelis on unikaalsuse kitsendus?',
                name='isikud',
                constraint_type='UNIQUE',
                points=1,
            ),
            ConstraintTest(
                title='Kas tabelis on check kitsendus?',
                name='isikud',
                constraint_type='CHECK',
                points=1,
            ),
        ]
    ),
    ChecksLayer(
        title='Tabeli Klubid kontrollid',
        tests=[
            StructureTest(
                title='Kas veerg asula on olemas?',
                name='klubid',
                column_name='asula',
                points=1,
            ),
            StructureTest(
                title='Kas veerg asukoht on kustutatud?',
                name='klubid',
                column_name='asukoht',
                should_exist=False,
                points=1,
            ),
            StructureTest(
                title='Kas veerg toimumiskoht on kustutatud?',
                name='klubid',
                column_name='toimumiskoht',
                should_exist=False,
                points=1,
            ),
        ]
    ),
    ChecksLayer(
        title='Tabeli Turniirid kontrollid',
        tests=[
           DataTest(
                title='Kas veerg asula on olemas?',
                name='turniirid',
                column_name='asula',
                points=1,
            ),
            ConstraintTest(
                title='Kas tabelis on välisvõti olemas?',
                name='turniirid',
                constraint_type='FOREIGN KEY',
                points=1,
            ), 
        ]
    ), 
    #TitleLayer('Kodutöö 4'),
    ChecksLayer(
        title='Vaate v_turniiripartiid kontrollid',
        tests=[
            StructureTest(
                title='Kas vaade v_turniiripartiid on olemas?',
                name='v_turniiripartiid',
                points=1,
            ),
            StructureTest(
                title='Kas veerg turniir_nimi on olemas?',
                name='v_turniiripartiid',
                column_name='turniir_nimi',
                points=1,
            ),
            StructureTest(
                title='Kas veerg toimumiskoht on olemas?',
                name='v_turniiripartiid',
                column_name='toimumiskoht',
                points=1,
            ),
            StructureTest(
                title='Kas veerg partii_id on olemas?',
                name='v_turniiripartiid',
                column_name='partii_id',
                points=1,
            ),
            StructureTest(
                title='Kas veerg partii_algus on olemas?',
                name='v_turniiripartiid',
                column_name='partii_algus',
                points=1,
            ),
            StructureTest(
                title='Kas veerg partii_lopp on olemas?',
                name='v_turniiripartiid',
                column_name='partii_lopp',
                points=1,
            ),
            StructureTest(
                title='Kas veerg kes_voitis on olemas?',
                name='v_turniiripartiid',
                column_name='kes_voitis',
            ),
            DataTest(
                title='Kas vaate andmete kogus on õige?',
                name='v_turniiripartiid',
                column_name='COUNT(*)',
                expected_value=299,
                points=1,
            ),
            DataTest(
                title='Kas partii 270 võitja tulemus on õige?',
                name='v_turniiripartiid',
                column_name='LOWER(kes_voitis)',
                where='partii_id = 270',
                expected_value='valge',
                points=1,
            ),
            DataTest(
                title='Kas partii 241 võitja tulemus on õige?',
                name='v_turniiripartiid',
                column_name='LOWER(kes_voitis)',
                where='partii_id = 241',
                expected_value='must',
                points=1,
            ),
            DataTest(
                title='Kas partii 193 sai viigi?',
                name='v_turniiripartiid',
                column_name='LOWER(kes_voitis)',
                where='partii_id = 193',
                expected_value='viik',
                points=1,
            ),
            
        ]
    ),
    ChecksLayer(
        title='Vaate v_klubipartiikogused kontrollid',
        tests=[
            StructureTest(
                title='Kas vaade v_klubipartiikogused on olemas?',
                name='v_klubipartiikogused',
                points=1,
            ),
            StructureTest(
                title='Kas veerg klubi_nimi on olemas?',
                name='v_klubipartiikogused',
                column_name='klubi_nimi',
                points=1,
            ),
            StructureTest(
                title='Kas veerg partiisid on olemas?',
                name='v_klubipartiikogused',
                column_name='partiisid',
                points=1,
            ),
            DataTest(
                title='Kas vaate andmete kogus on õige?',
                name='v_klubipartiikogused',
                column_name='COUNT(*)',
                expected_value=12,
                points=1,
            ),
            DataTest(
                title='Kas vaate partiide summa kokku on õige?',
                name='v_klubipartiikogused',
                column_name='SUM(partiisid)',
                expected_value=571,
                points=1,
            ), 
        ]
    ),
    
    ChecksLayer(
        title='Vaate v_keskminepartii kontrollid',
        tests=[
            StructureTest(
                title='Kas vaade v_keskminepartii on olemas?',
                name='v_keskminepartii',
                points=1,
            ),
            StructureTest(
                title='Kas veerg turniiri_nimi on olemas?',
                name='v_keskminepartii',
                column_name='turniiri_nimi',
                points=1,
            ),
            StructureTest(
                title='Kas veerg keskmine_partii on olemas?',
                name='v_keskminepartii',
                column_name='keskmine_partii',
                points=1,
            ),
            DataTest(
                title='Kas vaate andmete kogus on õige?',
                name='v_keskminepartii',
                column_name='COUNT(*)',
                expected_value=5,
                points=1,
            ),
            DataTest(
                title='Kas turniiril Plekkkarikas 2010 keskmine partii arv on õige?',
                name='v_keskminepartii',
                column_name='ROUND(keskmine_partii, 3)',
                where="turniiri_nimi = 'Plekkkarikas 2010'",
                expected_value=23.765,
                points=1,
            ),
            DataTest(
                title='Kas turniiril Kolme klubi kohtumine keskmine partii arv on õige?',
                name='v_keskminepartii',
                column_name='ROUND(keskmine_partii, 3)',
                where="turniiri_nimi = 'Kolme klubi kohtumine'",
                expected_value=23.040,
                points=1,
            ),
        ]
    ),
    ChecksLayer(
        title='Materialiseeritud vaate mv_partiide_arv_valgetega kontrollid',
        tests=[
            DataTest(
                title='Kas vaade mv_partiide_arv_valgetega on olemas?',
                name='mv_partiide_arv_valgetega',
                column_name='COUNT(*)',
                expected_value=85,
                points=1,
            ),
            DataTest(
                title='Kas leidub partii ühe mängijaga?',
                name='mv_partiide_arv_valgetega',
                column_name='COUNT(*)',
                where="eesnimi = 'Tarmo' AND perenimi = 'Kooser'",
                points=1,
            ),
            DataTest(
                title='Kas vaate kõige väikseim väärtus on õige?',
                name='mv_partiide_arv_valgetega',
                column_name='MIN(partiisid_valgetega)',
                expected_value=0,
                points=1,
            ),
            DataTest(
                title='Kas vaate kõige suurem väärtus on õige?',
                name='mv_partiide_arv_valgetega',
                column_name='MAX(partiisid_valgetega)',
                expected_value=14,
                points=1,
            )
        ]
    ),
    
]



from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest

tests = [
    ChecksLayer(
        title='Praktikum 4 ülesande 5 päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg linn?',
                name='query_test',
                column_name='linn',
                points=20,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg kokku?',
                name='query_test',
                column_name='kokku',
                points=20,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg maksimaalne?',
                name='query_test',
                column_name='maksimaalne',
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=14,
                points=20,
            ),
            QueryDataTest(
                title='Kas järjestus on õige?',
                name='query_test',
                column_name='linn',
                where='test_id=2',
                expected_value="Lisbon",
                points=10,
            ),
            QueryDataTest(
                title='Kas Turinis on õige maksimaalne?',
                name='query_test',
                column_name='kokku',
                where="linn='Turin'",
                expected_value="4640",
                points=10,
            ),
        ]
    ),
]


from silmused.ChecksLayer import ChecksLayer
from silmused.ExecuteLayer import ExecuteLayer
from silmused.tests.DataTest import DataTest
from silmused.tests.StructureTest import StructureTest
from silmused.tests.ConstraintTest import ConstraintTest
from silmused.tests.FunctionTest import FunctionTest
from silmused.tests.IndexTest import IndexTest
from silmused.tests.ProcedureTest import ProcedureTest
from silmused.tests.TriggerTest import TriggerTest
from silmused.tests.ViewTest import ViewTest
# TODO Add PASS tests
# TODO Add Function tests
# TODO Add Index tests
# TODO Add Procedure tests
# TODO Add QueryData tests
# TODO Add QueryStructure tests
# TODO Add Trigger tests
# TODO Add View tests
tests = [
ChecksLayer(
        title='DataTest feedback tests',
        tests=[
            DataTest(
                title='table_not_expected_value_should_exist_negative_feedback',
                name='inimesed'
            ),
            DataTest(
                title='table_column_not_expected_value_should_exist_negative_feedback',
                name='inimesed',
                column_name='eesnimi',
            ),
            DataTest(
                title='table_not_expected_value_should_not_exist_negative_feedback',
                name='isikud',
                should_exist=False,
            ),
            DataTest(
                title='table_column_not_expected_value_should_not_exist_negative_feedback',
                name='isikud',
                column_name='eesnimi',
                should_exist=False,
            ),
            DataTest(
                title='table_column_not_expected_value_should_exist_negative_feedback',
                name='isikud',
                column_name='COUNT(*)',
                where="eesnimi = 'Tarmo' AND perenimi = 'Kooserkkkk'",
            ),
            DataTest(
                title='isikud count',
                name='isikud',
                column_name='COUNT(*)',
                expected_value=86,
            ),
            DataTest(
                title='asula id min',
                name='asulad',
                column_name='MIN(id)',
                expected_value=2,
            ),
            DataTest(
                title='asula id max',
                name='asulad',
                column_name='MAX(id)',
                expected_value=9,
            ),
            DataTest(
                title='expected_value_should_exist_negative_feedback',
                name='isikud',
                column_name='eesnimi',
                where="perenimi = 'Kooser' and sugu = 'm'",
                expected_value='Liina',
            ),
            DataTest(
                title='expected_value_should_not_exist_negative_feedback',
                name='isikud',
                column_name='eesnimi',
                where="perenimi = 'Kooser' and sugu = 'm'",
                expected_value='Tarmo',
                should_exist=False
            ),
        ]
    ),
    # TODO Need more Constraint tests
    ChecksLayer(
        title='ConstraintTest feedback tests',
        tests=[
            ConstraintTest(
                title='multi_column_name_negative_feedback',
                name='inimesed',
                column_name=['eesnimi', 'perenimi', 'synnipaev'],
            ),
            ConstraintTest(
                title='multi_column_constraint_name_negative_feedback',
                name='inimesed',
                column_name=['eesnimi', 'perenimi', 'synnipaev'],
                constraint_name='pk_inimesed',
            ),
            ConstraintTest(
                title='table_constraint_should_exist_negative_feedback',
                name='riigid2',
            ),
            ConstraintTest(
                title='table_column_constraint_should_exist_negative_feedback',
                name='klubid',
                column_name='nimi',
            ),
            ConstraintTest(
                title='table_column_constraint_name_should_exist_negative_feedback',
                name='asulad',
                column_name='id',
                constraint_name='un1_isikukood',
            ),
            ConstraintTest(
                title='table_column_constraint_type_should_exist_negative_feedback',
                name='asulad',
                column_name='id',
                constraint_type='UNIQUE',
            ),
            ConstraintTest(
                title='table_column_constraint_name_and_type_should_exist_negative_feedback',
                name='asulad',
                column_name='id',
                constraint_name='un1_isikukood',
                constraint_type='UNIQUE',
            ),
            ConstraintTest(
                title='table_constraint_should_not_exist_negative_feedback',
                name='riigid',
                should_exist=False,
            ),
            ConstraintTest(
                title='table_column_constraint_should_not_exist_negative_feedback',
                name='asulad',
                column_name='nimi',
                should_exist=False,
            ),
            ConstraintTest(
                title='table_column_constraint_name_should_not_exist_negative_feedback',
                name='isikud',
                column_name='isikukood',
                constraint_name='un_isikukood',
                should_exist=False,
            ),
            ConstraintTest(
                title='table_column_constraint_type_should_not_exist_negative_feedback',
                name='isikud',
                column_name='isikukood',
                constraint_type='UNIQUE',
                should_exist=False,
            ),
            ConstraintTest(
                title='table_column_constraint_name_and_type_should_not_exist_negative_feedback',
                name='isikud',
                column_name='isikukood',
                constraint_name='un_isikukood',
                constraint_type='UNIQUE',
                should_exist=False,
            ),
        ]
    ),
    ChecksLayer(
        title='StructureTest feedback tests',
        tests=[
            StructureTest(
                title='expected_character_maximum_length_type_check_negative_feedback',
                name='isikud',
                column_name='eesnimi',
                expected_type='varchar',
                expected_character_maximum_length=80,
            ),
            StructureTest(
                title='expected_type_check_negative_feedback',
                name='isikud',
                column_name='eesnimi',
                expected_type='integer',
            ),
            StructureTest(
                title='table_should_exist_negative_feedback',
                name='isikud2',
            ),
            StructureTest(
                title='column_should_exist_negative_feedback',
                name='isikud',
                column_name='idd',
            ),
            StructureTest(
                title='table_should_not_exist_negative_feedback',
                name='isikud',
                should_exist=False,
            ),
            StructureTest(
                title='column_should_not_exist_negative_feedback',
                name='isikud',
                column_name='id',
                should_exist=False,
            ),
            # TODO are these StructureTest types necessary
            # Structure tests that seem pointless
            # StructureTest(
            #    title='expected_value_should_exist_negative_feedback',
            #    name='isikud',
            #    expected_value=80,
            # ),
            # StructureTest(
            #    title='expected_value_should_not_exist_negative_feedback',
            #    name='asulad',
            #    column_name='nimi',
            #    expected_value='Pärnu',
            #    should_exist=False
            # ),
        ]
    ),
]



from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest

tests = [
    ChecksLayer(
        title='Ülesanne 3 Päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg lavanimi?',
                name='query_test',
                column_name='lavanimi',
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg sünniriik?',
                name='query_test',
                column_name='sünniriik',
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg arv?',
                name='query_test',
                column_name='arv',
                points=10,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=17,
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige laulja 8. kohal?',
                name='query_test',
                column_name='lavanimi',
                where="test_id=8",
                expected_value='Lena',
                points=30,
            ),
            QueryDataTest(
                title='Kas on õige suurim osalemisarv?',
                name='query_test',
                column_name='arv',
                where="lavanimi='Valentina Monetta'",
                expected_value='4',
                points=20,
            ),
        ]
    ),
]


from silmused.ChecksLayer import ChecksLayer
from silmused.ExecuteLayer import ExecuteLayer
from silmused.tests.DataTest import DataTest
from silmused.tests.StructureTest import StructureTest
from silmused.tests.ConstraintTest import ConstraintTest
from silmused.tests.FunctionTest import FunctionTest
from silmused.tests.IndexTest import IndexTest
from silmused.tests.ProcedureTest import ProcedureTest
from silmused.tests.TriggerTest import TriggerTest
from silmused.tests.ViewTest import ViewTest
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest

"""
# Praktikum 3
tests = [
    # Ülesanne 1, 14; 10p
    ChecksLayer(
        title='Tabeli Turniirid kontrollid',
        tests=[
            # 1
            StructureTest(
                title='Kas veerg toimumiskoht on kustutatud?',
                name='turniirid',
                column_name='toimumiskoht',
                should_exist=False,
                points=2,
            ),
            # 1
            StructureTest(
                title='Kas veerg asukoht on olemas?',
                name='turniirid',
                column_name='asukoht',
                points=3,
                ),
            # 14
            ConstraintTest(
                title='Kas kitsendus ajakontroll on olemas?',
                name='turniirid',
                constraint_name='ajakontroll',
                points=5,
            ),
        ]
    ),
    # Ülesanne 2, 3, 4, 5 ja 8, 9, 12, 17, 18; 9 ül = 35p
    ChecksLayer(
        title='Tabeli Isikud kontrollid',
        tests=[
            # 2 ja 4
            StructureTest(
                title='Kas veerg Sisestatud on kustutatud?',
                name='isikud',
                column_name='sisestatud',
                should_exist=False,
                points=3,
            ),
            # 3
            ConstraintTest(
                title='Kas kitsendus un_isikukood on olemas?',
                name='isikud',
                column_name='isikukood',
                constraint_name='un_isikukood',
                constraint_type='UNIQUE',
                points=4,
            ),
            # 5
            ConstraintTest(
                title='Kas kitsendus isikud_un1 on kustutatud?',
                name='isikud',
                constraint_name='isikud_un1',
                constraint_type='UNIQUE',
                should_exist=False,
                points=2,
            ),
            # 8
            DataTest(
                title='Kas isiku Irys perenimi on muudetud?',
                name='isikud',
                column_name='perenimi',
                where="eesnimi = 'Irys'",
                expected_value='Kompvek',
                points=3,
            ),
            # 10
            DataTest(
                title='Kas P-tähega isikud on kustutatud?',
                name='isikud',
                column_name='COUNT(*)',
                where="eesnimi LIKE 'P%' AND RIGHT(eesnimi,1)='P'",
                expected_value=0,
                points=1,
            ),
            # 12
            StructureTest(
                title='Kas veerg klubis on olemas?',
                name='isikud',
                column_name='klubis',
                points=10,
            ),
            # 17
            DataTest(
                title='Kas Siim Susi on klubis Laudnikud?',
                name='isikud',
                column_name='COUNT(*)',
                where="eesnimi = 'Siim' AND perenimi = 'Susi' AND klubis = (select id from klubid where nimi = 'Laudnikud')",
                expected_value=1,
                points=5,
            ),
            # 18
            DataTest(
                title='Kas on lisatud 5 liiget klubwwisse Osav Oda?',
                name='isikud',
                column_name='COUNT(*)',
                where="klubis = (select id from klubid where nimi = 'Osav Oda')",
                expected_value=5,
                points=7,
            ),
        ]
    ),
    # 9; 15p
    ChecksLayer(
        title='Õ tähtede vahetamise kontrollid',
        tests=[
            DataTest(
                title='Kas isikute tabelis veerus eesnimi on "õ" tähed vahetatud?',
                name='isikud',
                column_name='eesnimi',
                where="eesnimi LIKE '%õ%'",
                points=5,
            ),
            DataTest(
                title='Kas isikute tabelis veerus perenimi on "õ" tähed vahetatud?',
                name='isikud',
                column_name='perenimi',
                where="perenimi LIKE '%õ%'",
                points=5,
            ),
            DataTest(
                title='Kas turniiride tabelis veerus nimi on "õ" tähed vahetatud?',
                name='turniirid',
                column_name='nimi',
                where="nimi LIKE '%õ%'",
                points=5,
            ),
        ],
    ),
    # Ülesanne 6, 13, 15; 20p
    ChecksLayer(
        title='Tabeli Partiid kontrollid',
        tests=[
            # 6
            ConstraintTest(
                title='Kas kitsendus vastavus on olemas?',
                name='partiid',
                constraint_name='vastavus',
                constraint_type='CHECK',
                points=5,
            ),
            # 13
            StructureTest(
                title='Kas veerg Kokkuvote on kustutatud?',
                name='partiid',
                column_name='kokkuvote',
                should_exist=False,
                points=5,
            ),
            # 15
            ConstraintTest(
                title='Kas kitsendus ajakontroll on olemas?',
                name='partiid',
                constraint_name='ajakontroll',
                points=10,
            ),
        ]
    ),
    # Ülesanne 7, 11, 16; 20p
    ChecksLayer(
        title='Tabeli Klubid kontrollid',
        tests=[
            # 7
            StructureTest(
                title='Kas veeru asukoha väärtuse maksimum suurus on õige?',
                name='klubid',
                column_name='asukoht',
                expected_character_maximum_length=100,
                points=5,
            ),
            # 11
            DataTest(
                title='Kas klubi osav oda on olemas?',
                name='klubid',
                where="nimi = 'Osav Oda'",
                points=10,
            ),
            # 16
            DataTest(
                title='Kas klubi Valge mask asukoht on Valga?',
                name='klubid',
                column_name='asukoht',
                where="nimi = 'Valge Mask'",
                expected_value='Valga',
                points=5,
            ),
        ]
    ),
]

# Praktikum 4

tests = [
    # Ülesanne 1, 4, 40p
    ChecksLayer(
        title='Tabeli Asulad kontrollid',
        tests=[
            StructureTest(
                title='Kas tabel asulad on olemas?',
                name='asulad',
                points=15,
            ),
            StructureTest(
                title='Kas veerg id on olemas?',
                name='asulad',
                column_name='id',
                points=3,
            ),
            StructureTest(
                title='Kas veerg nimi on olemas?',
                name='asulad',
                column_name='nimi',
                points=3,
            ),
            ConstraintTest(
                title='Kas tabeli primaarvõti on olemas?',
                name='asulad',
                constraint_type='PRIMARY KEY',
                points=2,
            ),
            ConstraintTest(
                title='Kas tabelis on unikaalsuse kitsendus?',
                name='asulad',
                constraint_type='UNIQUE',
                points=2,
            ),
            #4
            DataTest(
                title='Kas tabelis asulad on andmed olemas?',
                name='asulad',
                points=15,
            ),
        ]
    ),
    # Ülesanne 2, 3, 20p
    ChecksLayer(
        title='Tabeli Riigid kontrollid',
        tests=[
            StructureTest(
                title='Kas tabel riigid on olemas?',
                name='riigid',
                points=5,
            ),
            StructureTest(
                title='Kas veerg id on olemas?',
                name='riigid',
                column_name='id',
                points=1,
            ),
            StructureTest(
                title='Kas veerg nimi on olemas?',
                name='riigid',
                column_name='nimi',
                points=1,
            ),
            StructureTest(
                title='Kas veerg pealinn on olemas?',
                name='riigid',
                column_name='pealinn',
                points=1,
            ),
            StructureTest(
                title='Kas veerg rahvaarv on olemas?',
                name='riigid',
                column_name='rahvaarv',
                points=1,
            ),
            StructureTest(
                title='Kas veerg pindala on olemas?',
                name='riigid',
                column_name='pindala',
                points=1,
            ),
            StructureTest(
                title='Kas veerg skp_mld on olemas?',
                name='riigid',
                column_name='skp_mld',
                points=1,
            ),
            ConstraintTest(
                title='Kas tabeli primaarvõti on olemas?',
                name='asulad',
                constraint_type='PRIMARY KEY',
                points=2,
            ),
            ConstraintTest(
                title='Kas tabelis on unikaalsuse kitsendus?',
                name='asulad',
                constraint_type='UNIQUE',
                points=2,
            ),
            #3
            DataTest(
                title='Kas tabelis riigid on andmed olemas?',
                name='riigid',
                points=5,
            ),
        ]
    ),
    # Ülesanne 5, 6, 7, 9, 40p
    ChecksLayer(
        title='Tabeli Klubid kontrollid',
        tests=[
            #5
            StructureTest(
                title='Kas veerg asula on olemas?',
                name='klubid',
                column_name='asula',
                points=10,
            ),
            #6
            DataTest(
                title='Kas veerus asula on andmed olemas?',
                name='klubid',
                column_name='asula',
                points=10,
            ),
            #7
            ConstraintTest(
                title='Kas tabelis on välisvõti olemas?',
                name='klubid',
                constraint_type='FOREIGN KEY',
                points=10,
            ),
            #9
            StructureTest(
                title='Kas veerg asukoht on kustutatud?',
                name='klubid',
                column_name='asukoht',
                should_exist=False,
                points=5,
            ),
            #9
            StructureTest(
                title='Kas veerg toimumiskoht on kustutatud?',
                name='klubid',
                column_name='toimumiskoht',
                should_exist=False,
                points=5,
            ),

        ]
    ),
]

# Praktikum 8 100p
tests = [
    # 4p
    ChecksLayer(
        title='Tabeli Institutes kontrollid',
        tests=[
            StructureTest(
                title='Kas tabel institutes on olemas?',
                name='institutes',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='institutes',
                points=1,
            ),
            ConstraintTest(
                title='Kas välisvõti on olemas?',
                name='institutes',
                constraint_type='FOREIGN KEY',
                points=1,
            ),
            ConstraintTest(
                title='Kas välisvõti on olemas?',
                name='institutes',
                constraint_type='FOREIGN KEY',
                points=1,
            ),
        ]
    ),
    # 5p
    ChecksLayer(
        title='Tabeli Persons kontrollid',
        tests=[
            StructureTest(
                title='Kas tabel persons on olemas?',
                name='persons',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='persons',
                points=1,
            ),
            ConstraintTest(
                title='Kas välisvõti on olemas?',
                name='persons',
                constraint_type='FOREIGN KEY',
                points=3,
            ),
        ]
    ),
    # 4p
    ChecksLayer(
        title='Tabeli Registrations kontrollid',
        tests=[
            StructureTest(
                title='Kas tabel registrations on olemas?',
                name='registrations',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='registrations',
                points=1,
            ),
            ConstraintTest(
                title='Kas välisvõti on olemas?',
                name='registrations',
                constraint_type='FOREIGN KEY',
                points=1,
            ),
            ConstraintTest(
                title='Kas välisvõti on olemas?',
                name='registrations',
                constraint_type='FOREIGN KEY',
                points=1,
            ),
        ]
    ),
    # 3p
    ChecksLayer(
        title='Tabeli Lecturers kontrollid',
        tests=[
            StructureTest(
                title='Kas tabel lecturers  on olemas?',
                name='lecturers',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='lecturers',
                points=1,
            ),
            ConstraintTest(
                title='Kas välisvõti on olemas?',
                name='lecturers',
                constraint_type='FOREIGN KEY',
                points=1,
            ),
        ]
    ),
    # 5p
    ChecksLayer(
        title='Tabeli Courses kontrollid',
        tests=[
            StructureTest(
                title='Kas tabel courses on olemas?',
                name='courses',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='courses',
                points=1,
            ),
            ConstraintTest(
                title='Kas välisvõti on olemas?',
                name='courses',
                constraint_type='FOREIGN KEY',
                points=3,
            ),
        ],
    ),
    # 2p
    ChecksLayer(
        title='Vaate v_oigusteaduskonna_inimesed kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_oigusteaduskonna_inimesed on olemas?',
                name='v_oigusteaduskonna_inimesed',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_oigusteaduskonna_inimesed',
                column_name='COUNT(*)',
                expected_value=28,
                isView=True,
                points=1,
            ),
        ],
    ),
    # 2p
    ChecksLayer(
        title='Vaate v_persons_institute kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_persons_institute on olemas?',
                name='v_persons_institute',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_persons_institute',
                column_name='COUNT(*)',
                expected_value=300,
                isView=True,
                points=1,
            ),
        ],
    ),
    # 3p
    ChecksLayer(
        title='Vaate v_institute_deans kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_institute_deans on olemas?',
                name='v_institute_deans',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_institute_deans',
                column_name='COUNT(*)',
                expected_value=10,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas Usuteaduskonna vicedean on puudu?',
                name='v_institute_deans',
                column_name='vicedeanname',
                where="institutename='Usuteaduskond'",
                expected_value='None',
                isView=True,
                points=1,
            ),
        ],
    ),
    # 4p
    ChecksLayer(
        title='Vaate v_top20students kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_top20students on olemas?',
                name='v_top20students',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_top20students',
                column_name='COUNT(*)',
                expected_value=20,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas Henry Lesmenti eksami keskmine hinne on 4?',
                name='v_top20students',
                column_name='ROUND(averagegrade,0)',
                where="firstname='Henry' AND lastname = 'Lesment'",
                expected_value='4',
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas Mart Mustika eksami keskmine hinne on 3.8?',
                name='v_top20students',
                column_name='ROUND(averagegrade,1)',
                where="firstname='Mart' AND lastname = 'Mustikas'",
                expected_value='3.8',
                isView=True,
                points=1,
            ),
        ],
    ),
    # 2p - total 34p
    ChecksLayer(
        title='Kursus Sissejuhatus informaatikasse',
        tests=[
            DataTest(
                title='Kas kursus Sissejuhatus informaatikasse on olemas?',
                name='courses',
                column_name='COUNT(*)',
                where="name='Sissejuhatus informaatikasse'",
                expected_value=1,
                points=1,
            ),
            DataTest(
                title='Kas kursusel Sissejuhatus informaatikasse on tudengid olemas?',
                name='registrations',
                column_name='COUNT(*)',
                where="courseid=101",
                expected_value=15,
                points=1,
            ),
        ],
    ),
    # 13p
    ChecksLayer(
        title='Vaate v_persons_atleast_4eap kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_persons_atleast_4eap on olemas?',
                name='v_persons_atleast_4eap',
                points=5,
            ),
            ViewTest(
                title='Kas veerg FirstName on olemas?',
                name='v_persons_atleast_4eap',
                column_name='firstname',
                points=1,
            ),
            ViewTest(
                title='Kas veerg LastName on olemas?',
                name='v_persons_atleast_4eap',
                column_name='lastname',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_persons_atleast_4eap',
                column_name='COUNT(*)',
                expected_value=135,
                isView=True,
                points=6,
            ),
        ],
    ),
    # 23p
    ChecksLayer(
        title='Vaate v_countOfA kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_countOfA on olemas?',
                name='v_countofa',
                points=5,
            ),
            ViewTest(
                title='Kas veerg FirstName on olemas?',
                name='v_countofa',
                column_name='firstname',
                points=1,
            ),
            ViewTest(
                title='Kas veerg LastName on olemas?',
                name='v_countofa',
                column_name='lastname',
                points=1,
            ),
            ViewTest(
                title='Kas veerg CountOfA on olemas?',
                name='v_countofa',
                column_name='countofa',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_countofa',
                column_name='COUNT(*)',
                expected_value=119,
                isView=True,
                points=5,
            ),
            DataTest(
                title='Kas suurim A-de kogus on 5?',
                name='v_countofa',
                column_name='MAX(countofa)',
                expected_value=5,
                isView=True,
                points=5,
            ),
            DataTest(
                title='Kas väikseim A-de kogus on 1?',
                name='v_countofa',
                column_name='MIN(countofa)',
                expected_value=1,
                isView=True,
                points=5,
            ),
        ],
    ),
    # 30p
    ChecksLayer(
        title='Vaate v_top40A kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_top40A on olemas?',
                name='v_top40a',
                points=10,
            ),
            ViewTest(
                title='Kas veerg FirstName on olemas?',
                name='v_top40a',
                column_name='firstname',
                points=1,
            ),
            ViewTest(
                title='Kas veerg LastName on olemas?',
                name='v_top40a',
                column_name='lastname',
                points=1,
            ),
            ViewTest(
                title='Kas veerg CountOfA on olemas?',
                name='v_top40a',
                column_name='countofa',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_top40a',
                column_name='COUNT(*)',
                expected_value=40,
                isView=True,
                points=2,
            ),
            DataTest(
                title='Kas suurim A-de kogus on 6?',
                name='v_top40a',
                column_name='MAX(countofa)',
                expected_value=6,
                isView=True,
                points=10,
            ),
            DataTest(
                title='Kas väikseim A-de kogus on 4?',
                name='v_top40a',
                column_name='MIN(countofa)',
                expected_value=4,
                isView=True,
                points=5,
            ),
        ],
    ),
]"""
"""
# Praktikum 9 100p
tests = [
    # 1; 2p
    ChecksLayer(
        title='Vaate v_isikudklubid kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_isikudklubid on olemas?',
                name='v_isikudklubid',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_isikudklubid',
                isView=True,
                points=1,
            )
        ],
    ),
    # 2; 2p
    ChecksLayer(
        title='Vaate v_partiid kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_partiid on olemas?',
                name='v_partiid',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_partiid',
                isView=True,
                points=1,
            )
        ],
    ),
    # 5; 11p
    ChecksLayer(
        title='Vaate v_partiidpisi kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_isikudklubid on olemas?',
                name='v_partiidpisi',
                points=1,
            ),
            ViewTest(
                title='Kas veerg id on olemas?',
                name='v_partiidpisi',
                column_name='id',
                points=1,
            ),
            ViewTest(
                title='Kas veerg valge_mangija on olemas?',
                name='v_partiidpisi',
                column_name='valge_mangija',
                points=1,
            ),
            ViewTest(
                title='Kas veerg valge_punkt on olemas?',
                name='v_partiidpisi',
                column_name='valge_punkt',
                points=1,
            ),
            ViewTest(
                title='Kas veerg must_mangija on olemas?',
                name='v_partiidpisi',
                column_name='must_mangija',
                points=1,
            ),
            ViewTest(
                title='Kas veerg must_punkt on olemas?',
                name='v_partiidpisi',
                column_name='must_punkt',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_partiidpisi',
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas partiis 1 on valge ja musta mängijate nimed õigel kujul?',
                name='v_partiidpisi',
                column_name='COUNT(*)',
                where="id = 1 and valge_mangija = 'Katrin Kask' and must_mangija = 'Malle Maasikas'",
                expected_value=1,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas partiis 61 valge mängija sai 1 punkti?',
                name='v_partiidpisi',
                column_name='round(valge_punkt,0)',
                where="id = 61 and valge_mangija = 'Jelena Pirn'",
                expected_value=1,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas partiis 61 must mängija sai 0 punkti?',
                name='v_partiidpisi',
                column_name='round(must_punkt,0)',
                where="id = 61 and must_mangija = 'Henno Hiis'",
                expected_value=0,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas partiis 83 saadi viigi eest 0.5 punkti?',
                name='v_partiidpisi',
                column_name='COUNT(*)',
                where="id = 83 and must_punkt = 0.5 and valge_punkt = 0.5",
                expected_value=1,
                isView=True,
                points=1,
            ),
        ],
    ),
    # 6; 20p - total 35
    ChecksLayer(
        title='Vaate v_punktid kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_punktid on olemas?',
                name='v_punktid',
                points=1,
            ),
            ViewTest(
                title='Kas veerg partii on olemas?',
                name='v_punktid',
                column_name='partii',
                points=1,
            ),
            ViewTest(
                title='Kas veerg turniir on olemas?',
                name='v_punktid',
                column_name='turniir',
                points=1,
            ),
            ViewTest(
                title='Kas veerg mangija on olemas?',
                name='v_punktid',
                column_name='mangija',
                points=1,
            ),
            ViewTest(
                title='Kas veerg varv on olemas?',
                name='v_punktid',
                column_name='varv',
                points=1,
            ),
            ViewTest(
                title='Kas veerg punkt on olemas?',
                name='v_punktid',
                column_name='punkt',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_punktid',
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas mängija on V värvi?',
                name='v_punktid',
                column_name='varv',
                where="partii=51 and mangija=198",
                expected_value='V',
                isView=True,
                points=2,
            ),
            DataTest(
                title='Kas partii kaotaja sai 0 punkti?',
                name='v_punktid',
                column_name='round(punkt,0)',
                where="partii=51 and mangija=198",
                expected_value=0,
                isView=True,
                points=3,
            ),
            DataTest(
                title='Kas mängija on M värvi?',
                name='v_punktid',
                column_name='varv',
                where="partii=51 and mangija=82",
                expected_value='M',
                isView=True,
                points=2,
            ),
            DataTest(
                title='Kas partii võitja sai 1 punkti?',
                name='v_punktid',
                column_name='round(punkt,0)',
                where="partii=51 and mangija=82",
                expected_value=1,
                isView=True,
                points=3,
            ),
            DataTest(
                title='Kas viigi korral saadi 0.5 punkti?',
                name='v_punktid',
                column_name='round(punkt,1)',
                where="partii=54 and mangija=75",
                expected_value=0.5,
                isView=True,
                points=3,
            ),
        ],
    ),
    # 7; 25p -- total 60
    ChecksLayer(
        title='Vaate v_edetabelid kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_edetabelid on olemas?',
                name='v_edetabelid',
                points=7,
            ),
            ViewTest(
                title='Kas veerg id on olemas?',
                name='v_edetabelid',
                column_name='id',
                points=1,
            ),
            ViewTest(
                title='Kas veerg mangija on olemas?',
                name='v_edetabelid',
                column_name='mangija',
                points=1,
            ),
            ViewTest(
                title='Kas veerg synniaeg on olemas?',
                name='v_edetabelid',
                column_name='synniaeg',
                points=1,
            ),
            ViewTest(
                title='Kas veerg ranking on olemas?',
                name='v_edetabelid',
                column_name='ranking',
                points=1,
            ),
            ViewTest(
                title='Kas veerg klubi on olemas?',
                name='v_edetabelid',
                column_name='klubi',
                points=1,
            ),
            ViewTest(
                title='Kas veerg turniir on olemas?',
                name='v_edetabelid',
                column_name='turniir',
                points=1,
            ),
            ViewTest(
                title='Kas veerg punkte on olemas?',
                name='v_edetabelid',
                column_name='punkte',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_edetabelid',
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas on turniiril 42 kõige suurem punktide arv õige?',
                name='v_edetabelid',
                column_name='round(MAX(punkte),0)',
                where='turniir=42',
                expected_value=7,
                isView=True,
                points=5,
            ),
            DataTest(
                title='Kas on turniiril 41 kõige väiksem punktide arv õige?',
                name='v_edetabelid',
                column_name='round(MIN(punkte),1)',
                where='turniir=41',
                expected_value=0.5,
                isView=True,
                points=5,
            ),
        ],
    ),
    # 10; 10p -- total 70p
    ChecksLayer(
        title='Vaate mv_edetabelid kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade mv_edetabelid on olemas?',
                name='mv_edetabelid',
                isMaterialized=True,
                points=5,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='mv_edetabelid',
                isView=True,
                points=5,
            )
        ],
    ),
    # 11 ja 12; 20p -- total 90
    ChecksLayer(
        title='Vaate v_klubi54 kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_klubi54 on olemas?',
                name='v_klubi54',
                points=5,
            ),
            ViewTest(
                title='Kas veerg eesnimi on olemas?',
                name='v_klubi54',
                column_name='eesnimi',
                points=1,
            ),
            ViewTest(
                title='Kas veerg perenimi on olemas?',
                name='v_klubi54',
                column_name='perenimi',
                points=1,
            ),
            ViewTest(
                title='Kas veerg synniaeg on olemas?',
                name='v_klubi54',
                column_name='synniaeg',
                points=1,
            ),
            ViewTest(
                title='Kas veerg ranking on olemas?',
                name='v_klubi54',
                column_name='ranking',
                points=1,
            ),
            ViewTest(
                title='Kas veerg klubi_id on olemas?',
                name='v_klubi54',
                column_name='klubi_id',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_klubi54',
                isView=True,
                points=5,
            ),
            #12 insert
            DataTest(
                title='Kas Piibe Leht on lisatud vaate kaudu?',
                name='v_klubi54',
                column_name='COUNT(*)',
                where="eesnimi = 'Piibe' and perenimi = 'Leht'",
                expected_value=1,
                isView=True,
                points=5,
            ),
        ],
    ),
    # 13; 10p -- total 100
    ChecksLayer(
        title='Vaate v_maletaht kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_maletaht on olemas?',
                name='v_maletaht',
                points=1,
            ),
            ViewTest(
                title='Kas veerg id on olemas?',
                name='v_maletaht',
                column_name='id',
                points=1,
            ),
            ViewTest(
                title='Kas veerg eesnimi on olemas?',
                name='v_maletaht',
                column_name='eesnimi',
                points=1,
            ),
            ViewTest(
                title='Kas veerg perenimi on olemas?',
                name='v_maletaht',
                column_name='perenimi',
                points=1,
            ),
            ViewTest(
                title='Kas veerg isikukood on olemas?',
                name='v_maletaht',
                column_name='isikukood',
                points=1,
            ),
            ViewTest(
                title='Kas veerg klubis on olemas?',
                name='v_maletaht',
                column_name='klubis',
                points=1,
            ),
            ViewTest(
                title='Kas veerg synniaeg on olemas?',
                name='v_maletaht',
                column_name='synniaeg',
                points=1,
            ),
            ViewTest(
                title='Kas veerg sugu on olemas?',
                name='v_maletaht',
                column_name='sugu',
                points=1,
            ),
            ViewTest(
                title='Kas veerg ranking on olemas?',
                name='v_maletaht',
                column_name='ranking',
                points=1,
            ),
            DataTest(
                title='Kas andmed on olemas?',
                name='v_maletaht',
                column_name='COUNT(*)',
                expected_value=9,
                isView=True,
                points=1,
            )
        ],
    ),
]"""
# praktikum 10
tests = [
    # 1, 10p
    ChecksLayer(
        title='Funktsiooni f_mangija_punktid_turniiril kontrollid',
        tests=[
            FunctionTest(
                title='Kas mängija punktid turniiril on õige?',
                name='f_mangija_punktid_turniiril',
                arguments=[92,42],
                number_of_parameters=2,
                expected_value=5.5,
                points=10,
            ),
        ],
    ),
    # 2.1 5p
    ChecksLayer(
        title='Funktsiooni f_nimi kontrollid',
        tests=[
            FunctionTest(
                title='Kas nimekuju on õige?',
                name='f_nimi',
                arguments=['test', 'kesk'],
                number_of_parameters=2,
                expected_value='kesk, test',
                points=5,
            ),
        ],
    ),
    # 2.2 5p
    ChecksLayer(
        title='Funktsiooni f_klubisuurus kontrollid',
        tests=[
            FunctionTest(
                title='Kas klubisuurus on õige?',
                name='f_klubisuurus',
                arguments=[50],
                number_of_parameters=1,
                expected_value=9,
                points=5,
            ),
        ],
    ),
    # 3, 10p (30p)
    ChecksLayer(
        title='Funktsiooni f_mangija_koormus kontrollid',
        tests=[
            FunctionTest(
                title='Kas mängija koormus on õige?',
                name='f_mangija_koormus',
                arguments=[73],
                number_of_parameters=1,
                expected_value=18,
                points=10,
            ),
        ],
    ),
    # 4, 10p (40p)
    ChecksLayer(
        title='Funktsiooni f_mangija_voite_turniiril kontrollid',
        tests=[
            FunctionTest(
                title='Kas mängija võitude arv on õige?',
                name='f_mangija_voite_turniiril',
                arguments=[197, 43],
                number_of_parameters=2,
                expected_value=3,
                points=5,
            ),
            FunctionTest(
                title='Kas mängija võitude arv on 0?',
                name='f_mangija_voite_turniiril',
                arguments=[75, 44],
                number_of_parameters=2,
                expected_value=0,
                points=5,
            ),
        ],
    ),
    # 5.1, 10 (50p)
    ChecksLayer(
        title='Funktsiooni f_mangija_viike_turniiril kontrollid',
        tests=[
            FunctionTest(
                title='Kas mängija viikide arv on õige?',
                name='f_mangija_viike_turniiril',
                arguments=[197, 43],
                number_of_parameters=2,
                expected_value=1,
                points=5,
            ),
            FunctionTest(
                title='Kas mängija viikide arv on 0?',
                name='f_mangija_viike_turniiril',
                arguments=[75, 44],
                number_of_parameters=2,
                expected_value=0,
                points=5,
            ),
        ],
    ),
    # 5.2, 10 (60p)
    ChecksLayer(
        title='Funktsiooni f_mangija_kaotusi_turniiril kontrollid',
        tests=[
            FunctionTest(
                title='Kas mängija kaotuste arv on õige?',
                name='f_mangija_kaotusi_turniiril',
                arguments=[75, 44],
                number_of_parameters=2,
                expected_value=2,
                points=5,
            ),
            FunctionTest(
                title='Kas mängija kaotuste arv on 0?',
                name='f_mangija_kaotusi_turniiril',
                arguments=[197, 43],
                number_of_parameters=2,
                expected_value=0,
                points=5,
            ),
        ],
    ),
    # 6, 10p (70p)
    ChecksLayer(
        title='Funktsiooni f_voit_viik_kaotus kontrollid',
        tests=[
            FunctionTest(
                title='Kas on õige tulemuste arv?',
                name='f_voit_viik_kaotus',
                arguments=[44],
                number_of_parameters=1,
                expected_count=63,
                points=4,
            ),
            FunctionTest(
                title='Kas mängijal on õige võitude arv?',
                name='f_voit_viik_kaotus',
                column_name='voite',
                where="id = 193",
                arguments=[44],
                number_of_parameters=1,
                expected_value=1,
                points=2,
            ),
            FunctionTest(
                title='Kas mängijal on õige viikide arv?',
                name='f_voit_viik_kaotus',
                column_name='viike',
                where="id = 193",
                arguments=[44],
                number_of_parameters=1,
                expected_value=1,
                points=2,
            ),
            FunctionTest(
                title='Kas mängijal on õige kaotuste arv?',
                name='f_voit_viik_kaotus',
                column_name='kaotusi',
                where="id = 193",
                arguments=[44],
                number_of_parameters=1,
                expected_value=0,
                points=2,
            ),
        ],
    ),
    # 7, 10p (80p)
    ChecksLayer(
        title='Protseduuri sp_uus_isik kontrollid',
        tests=[
            ProcedureTest(
                title='Kas uus isik sai lisatud?',
                name='sp_uus_isik',
                arguments=['Uus', 'Isik', 51, 'm'],
                number_of_parameters=4,
                pre_query="DELETE FROM isikud WHERE eesnimi = 'Uus' and perenimi = 'Isik'",
                after_query="SELECT * FROM isikud WHERE eesnimi = 'Uus' and perenimi = 'Isik' and klubis = 51",
                points=8,
            ),
            DataTest(
                title='Kas klubiliikmete arv suurenes?',
                name='isikud',
                column_name='COUNT(*)',
                where="klubis=51",
                expected_value=5,
                points=2,
            ),
            ExecuteLayer("DELETE FROM isikud WHERE eesnimi = 'Uus' and pernimi = 'Isik'"),
        ],
    ),
    # 8, 10p (90p)
    ChecksLayer(
        title='Funktsiooni f_klubiparimad kontrollid',
        tests=[
            FunctionTest(
                title='Kas on õige kirjete arv?',
                name='f_klubiparimad',
                arguments=['Areng'],
                number_of_parameters=1,
                expected_count=3,
                points=5,
            ),
            FunctionTest(
                title='Kas on õige mängija tulemus?',
                name='f_klubiparimad',
                column_name='ROUND(punktisumma,1)',
                arguments=['Areng'],
                number_of_parameters=1,
                where="isik = 'Põder, Priit'",
                expected_value=4.5,
                points=5,
            ),
        ],
    ),
    # 11, 10p, (100p)
    ChecksLayer(
        title='Funktsiooni f_infopump kontrollid',
        tests=[
            FunctionTest(
                title='Kas on õige kirjete arv?',
                name='f_infopump',
                expected_count=[105, 108],
                points=10,
            ),
        ]
    ),
]

"""
_user1 = 123456
_user2 = 123457
_partii_id = 123123
_asula_id = 200
_asula_nimi='Test_asula'
_klubi_id = 201
_klubi_nimi = 'Klubi kustutamiseks'
tests = [
    ChecksLayer(
        title='Trigeri tg_ajakontroll kontrollid',
        tests=[
            TriggerTest(
                title='Kas on õige definitsioon tg_ajakontroll?',
                name='tg_ajakontroll',
                arguments=['UPDATE', 'INSERT'],
                action_timing='BEFORE',
                points=10,
            ),
        ]
    ),
    ChecksLayer(
        title='Indeksi ix_nimed kontroll',
        tests=[
            IndexTest(
                title='Kas on olemas indeks ix_nimed?',
                name='ix_nimed',
                points=10,
            ),
        ]
    ),
    ChecksLayer(
        title='Trigeri tg_riigid kontrollid',
        tests=[
            TriggerTest(
                title='Kas on õige definitsioon tg_riigid?',
                name='tg_riigid',
                arguments=['INSERT'],
                action_timing='BEFORE',
                points=10,
            ),
            ExecuteLayer("ALTER TABLE public.riigid DISABLE TRIGGER ALL"),
            ExecuteLayer("ALTER TABLE public.riigid ENABLE TRIGGER tg_riigid"),
            ExecuteLayer(f"DELETE FROM public.riigid WHERE id in (3000, 3001)"),
            ExecuteLayer(f"INSERT INTO public.riigid (id, nimi, pealinn) VALUES (3000,'Bulgaaria1','Sofia1')"),
            ExecuteLayer(f"INSERT INTO public.riigid (id, nimi, pealinn) VALUES (3001,'Bulgaaria','Sofia2')"),

            DataTest(
                title='Kas uue nimega riik lisati?',
                name='riigid',
                where="id=3000",
                should_exist=True,
                points=10,
            ),
            DataTest(
                title='Kas sama nimega riiki ei lisatud?',
                name='riigid',
                where="id=3001",
                should_exist=False,
                points=10,
            ),
        ]
    ),
    ChecksLayer(
        title='Trigeri tg_ajakontroll1 kontrollid',
        tests=[
            TriggerTest(
                title='Kas on õige definitsioon tg_ajakontroll1?',
                name='tg_ajakontroll1',
                arguments=['UPDATE', 'INSERT'],
                action_timing='BEFORE',
                points=10,
            ),
            ExecuteLayer("ALTER TABLE public.partiid DISABLE TRIGGER ALL"),
            ExecuteLayer("ALTER TABLE public.partiid ENABLE TRIGGER tg_ajakontroll1"),
            ExecuteLayer("ALTER TABLE public.isikud DISABLE TRIGGER ALL"),
            ExecuteLayer(f"DELETE FROM public.isikud WHERE id in ({_user1},{_user2})"),
            ExecuteLayer(f"DELETE FROM public.partiid where id in ({_partii_id},{_partii_id+1})"),
            ExecuteLayer(f"INSERT INTO public.isikud (id, eesnimi, perenimi) VALUES ({_user1},'Man', 'Ka')"),
            ExecuteLayer(f"INSERT INTO public.isikud (id, eesnimi, perenimi) VALUES ({_user2},'Kan', 'Ma')"),
            ExecuteLayer(f"INSERT INTO public.partiid VALUES (44,'2023-03-22 17:45:24.000','2023-03-22 17:47:24.000',{_user1},{_user2},2,0, {_partii_id})", ),
            ExecuteLayer(f"INSERT INTO partiid (turniir, algushetk, valge, must, valge_tulemus, musta_tulemus) VALUES (41, '2005-01-12 08:05:00.000', 73, 92, 1, 1, {_partii_id+1})", ),
            DataTest(
                title='Kas partii lisamine õnnestus?',
                name='partiid',
                where=f"id={_partii_id}",
                points=10,
            ),
            DataTest(
                title='Kas partii lisamine ebaõnnestus?',
                name='partiid',
                where=f"id={_partii_id+1}",
                points=10,
                should_exist=False,
            ),
        ]
    ),
    ChecksLayer(
        title='Trigeri tg_kustuta_klubi kontrollid',
        tests=[
            TriggerTest(
                title='Kas on õige definitsioon tg_kustuta_klubi?',
                name='tg_kustuta_klubi',
                arguments=['DELETE'],
                action_timing='AFTER',
                points=20,
            ),
            ExecuteLayer("ALTER TABLE public.klubid DISABLE TRIGGER ALL"),
            ExecuteLayer("ALTER TABLE public.klubid ENABLE TRIGGER tg_kustuta_klubi"),
            ExecuteLayer("ALTER TABLE public.asulad DISABLE TRIGGER ALL"),
            ExecuteLayer(f"DELETE FROM public.asulad WHERE id in ({_asula_id})"),
            ExecuteLayer(f"DELETE FROM public.klubid WHERE id in ({_klubi_id})"),
            ExecuteLayer(f"INSERT INTO public.asulad VALUES ({_asula_id},{_asula_nimi})"),
            ExecuteLayer(f"INSERT INTO public.klubid (id, nimi, asula) VALUES ({_klubi_id},{_klubi_nimi},{_asula_id}"),
            ExecuteLayer(f"DELETE FROM klubid WHERE nimi={_klubi_nimi}"),
            DataTest(
                title='Kas klubi kustutamisel kustus ka asula?',
                name='asulad',
                where=f"nimi='{_asula_nimi}'",
                should_exist=False,
                points=1,
            )
        ]
    ),
    ChecksLayer(
        title='Trigeri tg_partiiaeg1 kontrollid',
        tests=[
            TriggerTest(
                title='Kas on õige definitsioon tg_partiiaeg1?',
                name='tg_partiiaeg1',
                arguments=['UPDATE', 'INSERT'],
                action_timing='BEFORE',
                points=12,
            ),
            ExecuteLayer("ALTER TABLE public.partiid DISABLE TRIGGER ALL"),
            ExecuteLayer("ALTER TABLE public.partiid ENABLE TRIGGER tg_partiiaeg1"),
            ExecuteLayer("ALTER TABLE public.isikud DISABLE TRIGGER ALL"),
            ExecuteLayer(f"DELETE FROM public.isikud WHERE id in ({_user1},{_user2})"),
            ExecuteLayer(f"INSERT INTO public.isikud (id, eesnimi, perenimi) VALUES ({_user1},'Man', 'Ka')"),
            ExecuteLayer(f"INSERT INTO public.isikud (id, eesnimi, perenimi) VALUES ({_user2},'Kan', 'Ma')"),
            ExecuteLayer(f"INSERT INTO Partiid(turniir, algushetk, valge, must, id) VALUES (41, '2005-01-22 08:04', {_user1},{_user2},{_partii_id+2})"),
            ExecuteLayer(f"INSERT INTO Partiid(turniir, algushetk, valge, must, id) VALUES (41, '2005-01-10 08:04', {_user1},{_user2},{_partii_id+3})"),
            ExecuteLayer(f"INSERT INTO Partiid(turniir, algushetk, lopphetk, valge, must, id) VALUES (41, '2005-01-12 08:04', '2005-01-22 09:10', {_user1},{_user2},{_partii_id+4})"),
            ExecuteLayer(f"INSERT INTO Partiid(turniir, algushetk, lopphetk, valge, must, id) VALUES (41, '2005-01-12 08:04', '2005-01-01 09:10', {_user1},{_user2},{_partii_id+5})"),
            DataTest(
                title='Kas partii lisamine ebaõnnestus, kui algusaeg enne turniiri?',
                name='partiid',
                where=f"id={_partii_id+2}",
                points=2,
                should_exist=False,
            ),
            DataTest(
                title='Kas partii lisamine ebaõnnestus, kui algusaeg peale turniiri?',
                name='partiid',
                where=f"id={_partii_id+3}",
                points=2,
                should_exist=False,
            ),
            DataTest(
                title='Kas partii lisamine ebaõnnestus, kui lõppaeg enne turniiri?',
                name='partiid',
                where=f"id={_partii_id+4}",
                points=2,
                should_exist=False,
            ),
            DataTest(
                title='Kas partii lisamine ebaõnnestus, kui lõppaeg peale turniiri?',
                name='partiid',
                where=f"id={_partii_id+5}",
                points=2,
                should_exist=False,
            ),
        ]
    ),
]
"""
"""
tests = [
    ChecksLayer(
        title='Tabeli Tooted kontroll',
        tests=[
            StructureTest(
                title='Kas on olemas tabel Tooted?',
                name='tooted',
                points=30,
            ),
            StructureTest(
                title='Kas on olemas veerg id?',
                name='tooted',
                column_name='id',
                points=20,
            ),
            StructureTest(
                title='Kas on olemas veerg nimi?',
                name='tooted',
                column_name='nimi',
                points=20,
            ),
            DataTest(
                title='Kas on lisatud toode tugitool?',
                name='tooted',
                column_name='nimi',
                where="lower(nimi)='tugitool'",
                points=30,
            )
        ]
    )
]


tests=[
    ChecksLayer(
        title='Rollid ja userid',
        tests=[
            QueryDataTest(
                name='information_schema.role_column_grants',
                column_name='grantee',
                where="grantee='tudeng'",
                expected_value='tudeng',
            ),
            QueryDataTest(
                name='information_schema.role_column_grants',
                column_name='grantee',
                where="grantee='readonly'",
                expected_value='readonly',
            ),
        ]
    )
]
"""